
# This is not yet ready for use!
    

#==========================================
from __future__ import print_function
def plot_multiframes(plotdata, verbose=False):
#==========================================

    """
    Plot all figures that involve multiple frames

    """

    if plotdata.mode() == 'iplotclaw':
        pylab.ion()
        
    try:
        plotfigure_dict = plotdata.plotfigure_dict
    except:
        print('*** Error in plotframe: plotdata missing plotfigure_dict')
        print('*** This should not happen')
        return None

    if len(plotfigure_dict) == 0:
        print('*** Warning in plotframe: plotdata has empty plotfigure_dict')
        print('*** Apparently no figures to plot')


    # initialize current_data containing data that will be passed
    # to afterframe, afteraxes, afterpatch commands
    current_data = Data()
    current_data.user = Data()   # for user specified attributes
                                 # to avoid potential conflicts
    current_data.plotdata = plotdata


 
    if plotdata._mode == 'iplotclaw':
        print('    Plotting Frame %s at t = %s' % (frameno,t))
        requested_fignos = plotdata.iplotclaw_fignos
    else:
        requested_fignos = plotdata.print_fignos
    plotted_fignos = []

    plotdata = set_show(plotdata)   # set _show attributes for which figures
                                    # and axes should be shown.

    # loop over figures to look for multiframe plots
    # -------------------------------------------

    for figname in plotdata._fignames:
        plotfigure = plotdata.plotfigure_dict[figname]
        if not plotfigure._show:
            continue  # skip to next figure 

        figno = plotfigure.figno
        if requested_fignos != 'all':
            if figno not in requested_fignos:
                continue # skip to next figure

        plotted_fignos.append(figno)


        try:
            plotaxes_dict = plotfigure.plotaxes_dict
        except:
            print('*** Error in plotframe: plotdata missing plotaxes_dict')
            print('*** This should not happen')
            return  None

        if (len(plotaxes_dict) == 0) or (len(plotfigure._axesnames) == 0):
            print('*** Warning in plotframe: plotdata has empty plotaxes_dict')
            print('*** Apparently no axes to plot in figno ',figno)

        # loop over axes looking for type multiframe 
        # ------------------------------------------

        thisfigure_multiframe = False
        for axesname in plotfigure._axesnames:
            plotaxes = plotaxes_dict[axesname]
            if (plotaxes._show) and (plotaxes.type == 'multiframe'):
                thisfigure_multiframe = True
        if not thisfigure_multiframe:
            continue
        
        if 'facecolor' not in plotfigure.kwargs:
            # use Clawpack's default bg color (tan)
            plotfigure.kwargs['facecolor'] = '#ffeebb'   

        # create figure and set handle:
        plotfigure._handle = pylab.figure(num=figno, **plotfigure.kwargs)

        pylab.ioff()
        if plotfigure.clf_each_frame:
            pylab.clf()


        # loop over axes to appear on this figure:
        # ----------------------------------------

        for axesname in plotfigure._axesnames:
            plotaxes = plotaxes_dict[axesname]
            if (not plotaxes._show) or (plotaxes.type != 'multiframe'):
                continue   # skip this axes if no items show

            # create the axes:
            axescmd = getattr(plotaxes,'axescmd','subplot(1,1,1)')
            axescmd = 'plotaxes._handle = pylab.%s' % axescmd
            exec(axescmd)

            # loop over plotitems on these axes:
            # ----------------------------------

            for itemname in plotaxes._itemnames:
                
                plotitem = plotaxes.plotitem_dict[itemname]
                if plotitem._show == False:
                    print('Skipping item ', itemname)
                    continue
                if plotitem.plot_type not in ['1d_vs_t', '1d_xt']:
                    continue

                # Get solution for required frames:
                framenos = only_most_recent(plotitem.framenos)
                if len(framenos)==0:
                    print("*** No frames found in plot_multiframes for ")
                    print("*** plotitem named ",itemname)
                    continue
                elif plotitem.plot_type == '1d_xt':
                    plot_var = plotitem.plot_var
                    var_over_t = []
                    times = []
                    for frameno in framenos:
                        framesoln = plotdata.getframe(frameno, plotdata.outdir)
                        if len(framesoln.patches) > 1:
                            print("*** plot_type 1d_xt can't be used with AMR")
                            return None
                        patch = framesoln.patches[0]
                        thispatchvar = get_patchvar(patch,plot_var,1,current_data)
                        var_over_t.append(thispatchvar)
                        times.append(framesoln.t)
                    x = thispatchvar.xc_centers   # cell centers
                    var = array(var_over_t)
                    times = array(times)
                    contour(x,times,var)



            # set axes limits:
            if (plotaxes.xlimits is not None) & (type(plotaxes.xlimits) is not str):
                try:
                    pylab.xlim(plotaxes.xlimits[0], plotaxes.xlimits[1])
                except:
                    pass  # let axis be set automatically
            if (plotaxes.ylimits is not None) & (type(plotaxes.ylimits) is not str):
                try:
                    pylab.ylim(plotaxes.ylimits[0], plotaxes.ylimits[1])
                except:
                    pass  # let axis be set automatically

            if plotaxes.scaled:
                pylab.axis('scaled')


            # end of loop over plotaxes
            
        # end of loop over plotfigures


    # call an afterframe function if present:
    afterframe =  getattr(plotdata, 'afterframe', None)
    if afterframe:
        if isinstance(afterframe, str):
            # a string to be executed
            exec(afterframe)
        else:
            # assume it's a function
            try:
                output = afterframe(current_data)
                if output: current_data = output
            except:
                print('*** Error in afterframe ***')
                raise


    if plotdata.mode() == 'iplotclaw':
        pylab.ion()
    for figno in plotted_fignos:
        pylab.figure(figno)
        pylab.draw()

    if verbose:
        print('    Done with plotframe for frame %i at time %g' % (frameno,t))

    
    # print the figure(s) to file(s) if requested:
    if (plotdata.mode() != 'iplotclaw') & plotdata.printfigs:
        # iterate over all figures that are to be printed:
        for figno in plotted_fignos:
            printfig(frameno=frameno, figno=figno, \
                    format=plotdata.print_format, plotdir=plotdata.plotdir,\
                    verbose=verbose)

    return current_data

    # end of plotframe






import os, sys
from clawpack.visclaw import geoplot
from clawpack.geoclaw.nonuniform_grid_tools import make_mapc2p
import numpy


fname_celledges = os.path.abspath('celledges.data')


xlimits = [-300,50]

def setplot(plotdata=None):

    if plotdata is None:
        from clawpack.visclaw.data import ClawPlotData
        plotdata = ClawPlotData()

    plotdata.clearfigures()

    mapc2p, mx_edge, xp_edge = make_mapc2p(fname_celledges)

    def mapc2p_km(xc):
        x_m = mapc2p(xc)
        x_km = x_m / 1000.   # convert to km
        return x_km


    def add_annotation(current_data):
        from pylab import ticklabel_format, plot,grid,ones,sqrt, \
            legend,title,ylabel,text
        ticklabel_format(useOffset=False)

        hl = 3200.
        hr = 200.
        greens = (hl/hr)**(0.25)
        #print('greens = ',greens)
        #plot(current_data.x, greens*ones(current_data.x.shape),'g--')
        plot(xlimits,[greens,greens],'g--', label='$C_g$, Greens Law')
        ctrans = 2*sqrt(hl)/(sqrt(hl)+sqrt(hr))
        crefl = (sqrt(hl)-sqrt(hr))/(sqrt(hl)+sqrt(hr))
        #print('ctrans = ',ctrans)
        plot(xlimits,[ctrans,ctrans],'r--', label='$C_T$, Transmission coefficient')
        legend(loc='upper left')
        title('')
        ylabel('meters', fontsize=14)

        if current_data.frameno == 0:
            text(-80,-0.4,'$\longrightarrow$',fontsize=20)
            text(-80,-0.6,'Incident')
        h = current_data.q[0,:]
        mx2 = int(round(len(h)/2.))
        etamax2 = (h[:mx2] - hl).max()
        #print('mx2 = %i, etamax2 = %g' % (mx2,etamax2))

    plotfigure = plotdata.new_plotfigure(name='domain', figno=0)
    plotfigure.kwargs = {'figsize':(7,6.5)}
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.axescmd = 'axes([.1,.4,.8,.5])' #'subplot(211)'
    plotaxes.xlimits = xlimits
    #plotaxes.xlimits = [-100e3,-20e3]
    plotaxes.ylimits = [-1,3]
    plotaxes.title = 'Surface displacement'
    plotaxes.afteraxes = add_annotation

    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = geoplot.surface
    plotitem.color = 'b'
    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p_km


    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.show = False
    plotitem.plot_var = 1
    plotitem.color = 'k'
    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p_km

    plotaxes = plotfigure.new_plotaxes()
    plotaxes.axescmd = 'axes([.1,.1,.8,.2])' #'subplot(212)'
    plotaxes.xlimits = xlimits

    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = geoplot.topo
    plotitem.color = 'g'
    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p_km

    def fix_topo_plot(current_data):
        from pylab import title,xlabel
        title('')
        xlabel('kilometers', fontsize=14)
    plotaxes.afteraxes = fix_topo_plot

    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p_km



    #-----------------------------------------
    # Figures for gauges
    #-----------------------------------------
    plotfigure = plotdata.new_plotfigure(name='q', figno=300, \
                                         type='each_gauge')
    plotfigure.clf_each_gauge = True

    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = 'auto'
    plotaxes.ylimits = 'auto'
    plotaxes.title = 'Eta'
    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = 2
    plotitem.plotstyle = 'b-'


    #-----------------------------------------

    # Parameters used only when creating html and/or latex hardcopy
    # e.g., via pyclaw.plotters.frametools.printframes:
    
    plotdata.printfigs = True                # print figures
    plotdata.print_format = 'png'            # file format
    plotdata.print_framenos = 'all'          # list of frames to print
    plotdata.print_gaugenos = 'all'          # list of gauges to print
    plotdata.print_fignos = 'all'            # list of figures to print
    plotdata.html = True                     # create html files of plots?
    plotdata.html_homelink = '../README.html'   # pointer for top of index
    plotdata.latex = True                    # create latex file of plots?
    plotdata.latex_figsperline = 2           # layout of plots
    plotdata.latex_framesperline = 1         # layout of plots
    plotdata.latex_makepdf = False           # also run pdflatex?
    plotdata.parallel = True                 # make multiple frame png's at once

    return plotdata




""" 
Set up the plot figures, axes, and items to be done for each frame.

This module is imported by the plotting routines and then the
function setplot is called to set the plot parameters.
    
""" 


from __future__ import absolute_import
from __future__ import print_function

try:
    from setplotfg import setplotfg
except:
    print("Did not find setplotfg.py")
    setplotfg = None


#--------------------------
def setplot(plotdata=None):
#--------------------------
    
    """ 
    Specify what is to be plotted at each frame.
    Input:  plotdata, an instance of pyclaw.plotters.data.ClawPlotData.
    Output: a modified version of plotdata.
    
    """ 

    if plotdata is None:
        from clawpack.visclaw.data import ClawPlotData
        plotdata = ClawPlotData()


    from clawpack.visclaw import colormaps, geoplot

    plotdata.clearfigures()  # clear any old figures,axes,items data

    plotdata.format = 'ascii'                # Format of output
    # plotdata.format = 'netcdf'             

    def set_drytol(current_data):
        # The drytol parameter is used in masking land and water and
        # affects what color map is used for cells with small water depth h.
        # The cell will be plotted as dry if h < drytol.
        # The best value to use often depends on the application and can
        # be set here (measured in meters):
        current_data.user['drytol'] = 1.e-2

    plotdata.beforeframe = set_drytol


    #-----------------------------------------
    # Figure for pcolor plot
    #-----------------------------------------
    plotfigure = plotdata.new_plotfigure(name='pcolor', figno=0)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes('pcolor')
    plotaxes.title = 'Surface'
    plotaxes.scaled = True

    # Water
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    #plotitem.plot_var = geoplot.surface
    plotitem.plot_var = geoplot.surface_or_depth
    plotitem.pcolor_cmap = geoplot.tsunami_colormap
    plotitem.pcolor_cmin = -0.9
    plotitem.pcolor_cmax = 0.9
    plotitem.add_colorbar = True
    plotitem.amr_celledges_show = [1,1,0]
    plotitem.amr_patchedges_show = [1]

    # Land
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = geoplot.land
    plotitem.pcolor_cmap = geoplot.land_colors
    plotitem.pcolor_cmin = 0.0
    plotitem.pcolor_cmax = 100.0
    plotitem.add_colorbar = False
    plotitem.amr_celledges_show = [1,1,0]
    plotaxes.xlimits = [-100,100]
    plotaxes.ylimits = [-100,100]



    #-----------------------------------------
    # Figure for zoom
    #-----------------------------------------
    plotfigure = plotdata.new_plotfigure(name='Zoom', figno=10)
    #plotfigure.show = False
    plotfigure.kwargs = {'figsize':[12,7]}

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes('diag zoom')
    plotaxes.axescmd = 'axes([0.0,0.1,0.6,0.6])'
    plotaxes.title = 'On diagonal'
    plotaxes.scaled = True
    plotaxes.xlimits = [55,66]
    plotaxes.ylimits = [55,66]

    def addgauges(current_data):
        from clawpack.visclaw import gaugetools
        gaugenos = range(101,110) # on diagonal
        gaugetools.plot_gauge_locations(current_data.plotdata, \
             gaugenos=gaugenos, format_string='ko', add_labels=True)
    
    plotaxes.afteraxes = addgauges

    # Water
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    #plotitem.plot_var = geoplot.surface
    plotitem.plot_var = geoplot.surface_or_depth
    plotitem.pcolor_cmap = geoplot.tsunami_colormap
    plotitem.pcolor_cmin = -0.9
    plotitem.pcolor_cmax = 0.9
    plotitem.add_colorbar = True
    plotitem.amr_celledges_show = [1,1,0]
    plotitem.amr_patchedges_show = [1]

    # Land
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = geoplot.land
    plotitem.pcolor_cmap = geoplot.land_colors
    plotitem.pcolor_cmin = 0.0
    plotitem.pcolor_cmax = 100.0
    plotitem.add_colorbar = False
    plotitem.amr_celledges_show = [1,1,0]

    # Add contour lines of bathymetry:
    plotitem = plotaxes.new_plotitem(plot_type='2d_contour')
    plotitem.plot_var = geoplot.topo
    from numpy import arange, linspace
    plotitem.contour_levels = arange(-10., 0., 1.)
    plotitem.amr_contour_colors = ['k']  # color on each level
    plotitem.kwargs = {'linestyles':'solid'}
    plotitem.amr_contour_show = [0,0,1]  # show contours only on finest level
    plotitem.celledges_show = 0
    plotitem.patchedges_show = 0
    plotitem.show = True

    # Add contour lines of topography:
    plotitem = plotaxes.new_plotitem(plot_type='2d_contour')
    plotitem.plot_var = geoplot.topo
    from numpy import arange, linspace
    plotitem.contour_levels = arange(0., 11., 1.)
    plotitem.amr_contour_colors = ['g']  # color on each level
    plotitem.kwargs = {'linestyles':'solid'}
    plotitem.amr_contour_show = [0,0,1]  # show contours only on finest level
    plotitem.celledges_show = 0
    plotitem.patchedges_show = 0
    plotitem.show = True

    # Add dashed contour line for shoreline
    plotitem = plotaxes.new_plotitem(plot_type='2d_contour')
    plotitem.plot_var = geoplot.topo
    plotitem.contour_levels = [0.]
    plotitem.amr_contour_colors = ['k']  # color on each level
    plotitem.kwargs = {'linestyles':'dashed'}
    plotitem.amr_contour_show = [0,0,1]  # show contours only on finest level
    plotitem.celledges_show = 0
    plotitem.patchedges_show = 0
    plotitem.show = True



    #-----------------------------------------
    # Figure for zoom near axis
    #-----------------------------------------
    #plotfigure = plotdata.new_plotfigure(name='Zoom2', figno=11)
    # now included in same figure as zoom on diagonal

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes('x zoom')
    plotaxes.show = True
    plotaxes.axescmd = 'axes([0.5,0.1,0.6,0.6])'
    plotaxes.title = 'On x-axis'
    plotaxes.scaled = True
    plotaxes.xlimits = [82,93]
    plotaxes.ylimits = [-5,6]

    def addgauges(current_data):
        from clawpack.visclaw import gaugetools
        gaugenos = range(1,10) # on x-axis
        gaugetools.plot_gauge_locations(current_data.plotdata, \
             gaugenos=gaugenos, format_string='ko', add_labels=True)
    
    plotaxes.afteraxes = addgauges

    # Water
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    #plotitem.plot_var = geoplot.surface
    plotitem.plot_var = geoplot.surface_or_depth
    plotitem.pcolor_cmap = geoplot.tsunami_colormap
    plotitem.pcolor_cmin = -0.9
    plotitem.pcolor_cmax = 0.9
    plotitem.add_colorbar = True
    plotitem.amr_celledges_show = [1,1,0]
    plotitem.amr_patchedges_show = [1]

    # Land
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = geoplot.land
    plotitem.pcolor_cmap = geoplot.land_colors
    plotitem.pcolor_cmin = 0.0
    plotitem.pcolor_cmax = 100.0
    plotitem.add_colorbar = False
    plotitem.amr_celledges_show = [1,1,0]


    # Add contour lines of bathymetry:
    plotitem = plotaxes.new_plotitem(plot_type='2d_contour')
    plotitem.plot_var = geoplot.topo
    from numpy import arange, linspace
    plotitem.contour_levels = arange(-10., 0., 1.)
    plotitem.amr_contour_colors = ['k']  # color on each level
    plotitem.kwargs = {'linestyles':'solid'}
    plotitem.amr_contour_show = [0,0,1]  # show contours only on finest level
    plotitem.celledges_show = 0
    plotitem.patchedges_show = 0
    plotitem.show = True

    # Add contour lines of topography:
    plotitem = plotaxes.new_plotitem(plot_type='2d_contour')
    plotitem.plot_var = geoplot.topo
    from numpy import arange, linspace
    plotitem.contour_levels = arange(0., 11., 1.)
    plotitem.amr_contour_colors = ['g']  # color on each level
    plotitem.kwargs = {'linestyles':'solid'}
    plotitem.amr_contour_show = [0,0,1]  # show contours only on finest level
    plotitem.celledges_show = 0
    plotitem.patchedges_show = 0
    plotitem.show = True

    # Add dashed contour line for shoreline
    plotitem = plotaxes.new_plotitem(plot_type='2d_contour')
    plotitem.plot_var = geoplot.topo
    plotitem.contour_levels = [0.]
    plotitem.amr_contour_colors = ['k']  # color on each level
    plotitem.kwargs = {'linestyles':'dashed'}
    plotitem.amr_contour_show = [0,0,1]  # show contours only on finest level
    plotitem.celledges_show = 0
    plotitem.patchedges_show = 0
    plotitem.show = True



    #-----------------------------------------
    # Figures for gauges
    #-----------------------------------------
    plotfigure = plotdata.new_plotfigure(name='Surface & topo', figno=300, \
                    type='each_gauge')

    plotfigure.clf_each_gauge = True

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = 'auto'
    plotaxes.ylimits = [-2.0, 2.0]
    plotaxes.title = 'Surface'

    # Plot surface as blue curve:
    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = 3
    plotitem.plotstyle = 'b-'

    # Plot topo as green curve:
    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')

    def gaugetopo(current_data):
        q = current_data.q
        h = q[0,:]
        eta = q[3,:]
        topo = eta - h
        return topo
        
    plotitem.plot_var = gaugetopo
    plotitem.plotstyle = 'g-'
    def add_zeroline(current_data):
        from pylab import plot, legend
        t = current_data.t
        legend(('surface','topography'),loc='lower left')
        plot(t, 0*t, 'k')

    plotaxes.afteraxes = add_zeroline


    #-----------------------------------------
    # Figure for patches alone
    #-----------------------------------------
    plotfigure = plotdata.new_plotfigure(name='patches', figno=2)
    plotfigure.show = False

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = [0,1]
    plotaxes.ylimits = [0,1]
    plotaxes.title = 'patches'
    plotaxes.scaled = True

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='2d_patch')
    plotitem.amr_patch_bgcolor = ['#ffeeee', '#eeeeff', '#eeffee']
    plotitem.amr_celledges_show = [1,1,0]   
    plotitem.amr_patchedges_show = [1]     

    #-----------------------------------------
    # Scatter plot of surface for radially symmetric
    #-----------------------------------------
    plotfigure = plotdata.new_plotfigure(name='Scatter', figno=200)
    plotfigure.show = False
    # Note: will not look very good unless more of domain is refined

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = [0., 100.]
    plotaxes.ylimits = [-1.5, 2.]
    plotaxes.title = 'Scatter plot of surface'

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='1d_from_2d_data')
    plotitem.plot_var = geoplot.surface
    def q_vs_radius(current_data):
        from numpy import sqrt
        x = current_data.x
        y = current_data.y
        r = sqrt(x**2 + y**2)
        q = current_data.var
        return r,q
    plotitem.map_2d_to_1d = q_vs_radius
    plotitem.plotstyle = 'o'
    plotitem.amr_color=['b','r','g']
    plotaxes.afteraxes = "import pylab; pylab.legend(['Level 1','Level 2'])"
    

    #-----------------------------------------
    
    # Parameters used only when creating html and/or latex hardcopy
    # e.g., via pyclaw.plotters.frametools.printframes:

    plotdata.printfigs = True                # print figures
    plotdata.print_format = 'png'            # file format
    plotdata.print_framenos = 'all'          # list of frames to print
    plotdata.print_gaugenos = [4,5,104,105]  # list of gauges to print
    plotdata.print_fignos = 'all'            # list of figures to print
    plotdata.html = True                     # create html files of plots?
    plotdata.html_homelink = '../README.html'   # pointer for top of index
    plotdata.latex = True                    # create latex file of plots?
    plotdata.latex_figsperline = 2           # layout of plots
    plotdata.latex_framesperline = 1         # layout of plots
    plotdata.latex_makepdf = False           # also run pdflatex?
    plotdata.parallel = True                 # make multiple frame png's at once
    plotdata.html_movie_width = 800         # width for js movie

    return plotdata

    



"""
Module Iplotclaw for interactive plotting of Clawpack results.

For more instructions see the Usage notes in class Iplotclaw below. 

For options during looping type:
  >>> ip = Iplotclaw()
  >>> ip.plotloop()
  PLOTCLAW> help

"""

import sys

if 'matplotlib' not in sys.modules:
    import matplotlib
    # Override system defaults before importing pylab
    # If you set the backend here, you must start ipython w/o pylab
    # before importing this package.
    #matplotlib.use('Agg')  # Use an image backend
    #matplotlib.use('TkAgg')
    matplotlib.rc('text', usetex=False)
    matplotlib.interactive(True)

from clawpack.visclaw import frametools
from .iplot import Iplot

#------------------------
class Iplotclaw(Iplot):
#------------------------
    """
    Class for interactively stepping through Clawpack plots.
    Uses frametools.plotframe to actually plot each frame.

    Usage:
    ------
    >>> from clawpack.visclaw.Iplotclaw import Iplotclaw 
    >>> ip = Iplotclaw()             # new instantiation
    >>> ip.plotloop()                # to start looping
    PLOTCLAW > help                  # for list of available commands
    PLOTCLAW > q                     # to quit looping and return to python
    >>> ip.plotloop()                # to restart looping at previous frame,
                                     #    with data, outdir, etc. preserved.
    Arguments:
    ----------
    The defaults are:
       Iplotclaw(setplot='setplot.py', outdir=None) 
    The setplot argument gives the file providing the setplot function.
    The outdir argument gives the directory providing the fort.* files.
      If outdir==None, the file .output created by 'make .output' is
      examined to determine where the most recent output might be found.
      If .output does not exist, outdir defaults to '.', the current directory.
    Other arguments of Iplotclaw rarely need to be changed:
       completekey='tab', stdin=None, stdout=None
    """

    # initialization:
    # ---------------

    prompt = 'PLOTCLAW > '
    lastcmd = 'n'             # initialize so <return> advances frame'

    def __init__(self, setplot='setplot.py', outdir=None, \
                 completekey='tab', stdin=None, stdout=None, simple=False,
                 controller=None):
        """Instantiate a line-oriented interpreter framework.


        The optional argument 'completekey' is the readline name of a
        completion key; it defaults to the Tab key. If completekey is
        not None and the readline module is available, command completion
        is done automatically. The optional arguments stdin and stdout
        specify alternate input and output file objects; if not specified,
        sys.stdin and sys.stdout are used.

        """
        from clawpack.visclaw import data

        self.simple = simple

        if stdin is not None:
            self.stdin = stdin
        else:
            self.stdin = sys.stdin
        if stdout is not None:
            self.stdout = stdout
        else:
            self.stdout = sys.stdout
        self.cmdqueue = []
        self.completekey = completekey
 
        self.setplot = setplot
        plotdata = data.ClawPlotData(controller)
        plotdata.setplot = self.setplot
        plotdata._mode = 'iplotclaw'

        if outdir is None:
            try:
                # possibly set by 'make .output' and stored in .output:
                dot_output = open('.output','r')    
                outdir = dot_output.readline().strip()
                dot_output.close()
            except:
                outdir = '.'
        plotdata.outdir = outdir
        # Note also that outdir might be reset by setplot!

        if not simple:
            try:
                plotdata = frametools.call_setplot(self.setplot,plotdata)
            except:
                print('*** Problem executing setplot in Iplotclaw')
                print('    setplot = ', self.setplot)
                print('*** Either this file does not exist or ')
                print('    there is a problem executing the function setplot in this file.')
                print('*** PLOT PARAMETERS MAY NOT BE SET! ***')
                raise

        self.plotdata = plotdata
        self.prevplotdata = plotdata
        self.restart = False
        self.prevframeno = 0

        self.frames = {}

    def plot_and_cache(self,frameno):
        try:
            if frameno not in list(self.frames.keys()):
                frametools.plotframe(self.frameno, self.plotdata, simple=self.simple, refresh=True)
                self.frames[str(frameno)] = ''
            else:
                frametools.plotframe(self.frameno, self.plotdata, simple=self.simple, refresh=False)
        except IOError as e:
            print(str(e))
    def help_n(self):
        print('n: advance to next frame\n')

    def load_frame(self,frameno):
        frame = self.plotdata.getframe(frameno, self.plotdata.outdir)
        return frame


    # Commands that can be typed at the PLOTCLAW> prompt:
    # ---------------------------------------------------

    # call setplot again
    # --------------------
    def do_resetplot(self, rest):
        if rest:
            self.setplot = rest
            print('*** Resetting setplot to: ',rest)
            self.plotdata.setplot = self.setplot
        print('Executing setplot from ',self.setplot)
        try:
            frametools.call_setplot(self.setplot,self.plotdata)
        except:
            print('*** Problem re-executing setplot')
            raise

    def help_resetplot(self):
        print('resetplot: re-execute the function setplot')
        print('           The easiest way to change plotting parameters')
        print('           is to modify setplot.py and then do resetplot.')
        print(' ')
        print('resetplot <new>: switch to a different setplot function')
        print('           as specified by <new>, which is a function or')
        print('           a string specifying the module containing setplot.')

    # show plot parameters:
    # ---------------------
    def do_show(self, rest):
        self.plotdata.showitems()

    def help_show(self):
        print('show: show the current plot items')


    # cleargauges
    # ---------
    def do_cleargauges(self, rest):
        if rest=='':
            self.plotdata.gaugesoln_dict.clear()
            print('Cleared all gauges')
        else:
            print('Not implemented: try cleargauges')

    def help_cleargauges(self):
        print('cleargauges: delete gauge data from cache to replot')
        print('    use if you have rerun the code and want to plot the')
        print('    latest results')


    # plotgauge commands:
    # --------------
    def do_plotgauge(self, rest):
        from clawpack.visclaw import gaugetools
        import os
        outdir = os.path.abspath(self.plotdata.outdir)

        # Construct gaugeno list
        if rest in ['','all']:
            try:
                import clawpack.amrclaw.data as amrclaw
            except ImportError as e:
                print("You must have AMRClaw installed to plot gauges.")
                print("continuing...")
                return
            
            gaugedata = amrclaw.GaugeData()
            gaugedata.read(self.plotdata.outdir)
            gaugenos = gaugedata.gauge_numbers
        else:
            gaugenos = [int(rest)]

        # Loop through requested gauges and read in if not in gaugesoln_dict
        for (n,gaugeno) in enumerate(gaugenos):
            # Is the next line necessary or can we delete it?
            gauge = self.plotdata.getgauge(gaugeno,outdir)
            
            gaugetools.plotgauge(gaugeno,self.plotdata)

            if n < len(gaugenos) - 1:
                ans = input("      Hit return for next gauge or q to quit ")
                if ans == "q":
                    break


    def help_plotgauge(self):
        print('plotgauge n  : plot figure for gauge number n, if found')
        print('plotgauge all: loop through plots of all gauges')
        print('plotgauge    : loop through plots of all gauges')
        
        
    # Convenience functions for examining solution or making additional
    # plots from the ipython command line:
    # -----------------------------------------------------------------

    def get_frame(self, frameno=None):
        """
        Return the framesoln for Frame for frameno.  
        If frameno is not specified, use the most recently plotted frameno.
        """
        if frameno is None:
            frameno = self.frameno

        return self.plotdata.getframe(frameno)

    def get_t(self, frameno=None):
        """
        Return the time for Frame for frameno.  
        If frameno is not specified, use the most recently plotted frameno.
        """
        if frameno is None:
            frameno = self.frameno

        return self.plotdata.getframe(frameno).t


    def get_patches(self, frameno=None):
        """
        Return the list of patches for frameno.  
        If frameno is not specified, use the most recently plotted frameno.
        """
        if frameno is None:
            frameno = self.frameno

        return self.plotdata.getframe(frameno).patches

    def get_patch(self, frameno=None):
        """
        Return the final patch for frameno.  
        If frameno is not specified, use the most recently plotted frameno.  
        If AMR is not used and there is only one patch, then return this one 
        (rather than a list with one element, as get_patches would return).  
        If AMR is used, then the final patch plotted is returned, 
        similar to claw/matlab/plotclaw behavior where only the final patch 
        is easily available after the plots are made.
        """
        if frameno is None:
            frameno = self.frameno
        
        return self.plotdata.getframe(frameno).patches[-1]

    def otherfigures(self):
        """
        Create any other figures specified in plotdata.otherfigure_dict.
        """
        
        plotdata = self.plotdata
        if len(plotdata.otherfigure_dict)==0:
            print("No other figures specified.")
        else:
            for name in plotdata.otherfigure_dict.keys():
                otherfigure = plotdata.otherfigure_dict[name]
                fname = otherfigure.fname
                makefig = otherfigure.makefig
                if makefig:
                    if type(makefig)==str:
                        try:
                            exec(makefig)
                        except:
                            print("*** Problem executing makefig ")
                            print("    for otherfigure ",name)
                            import pdb; pdb.set_trace()
                    else:
                        try:
                            makefig(plotdata)
                        except:
                            print("*** Problem executing makefig function")
                            print("    for otherfigure ",name)
                else:
                    print("No makefig function specified for ",name)


# end of Iplotclaw.
#----------------------------------------------------------------------




#!/usr/bin/env python
# encoding: utf-8
r"""
Two-dimensional variable-coefficient acoustics
==============================================

Solve the variable-coefficient acoustics equations in 2D:

.. math:: 
    p_t + K(x,y) (u_x + v_y) & = 0 \\ 
    u_t + p_x / \rho(x,y) & = 0 \\
    v_t + p_y / \rho(x,y) & = 0.

Here p is the pressure, (u,v) is the velocity, :math:`K(x,y)` is the bulk modulus,
and :math:`\rho(x,y)` is the density.

This example shows how to solve a problem with variable coefficients.
The left and right halves of the domain consist of different materials.
"""
 
import numpy as np

def setup(kernel_language='Fortran', use_petsc=False, outdir='./_output', 
          solver_type='classic', time_integrator='SSP104', lim_type=2, 
          disable_output=False, num_cells=(200, 200)):
    """
    Example python script for solving the 2d acoustics equations.
    """
    from clawpack import riemann

    if use_petsc:
        import clawpack.petclaw as pyclaw
    else:
        from clawpack import pyclaw

    if solver_type=='classic':
        solver=pyclaw.ClawSolver2D(riemann.vc_acoustics_2D)
        solver.dimensional_split=False
        solver.limiters = pyclaw.limiters.tvd.MC
    elif solver_type=='sharpclaw':
        solver=pyclaw.SharpClawSolver2D(riemann.vc_acoustics_2D)
        solver.time_integrator=time_integrator
        if time_integrator=='SSPLMMk2':
            solver.lmm_steps = 3
            solver.cfl_max = 0.25
            solver.cfl_desired = 0.24

    solver.bc_lower[0]=pyclaw.BC.wall
    solver.bc_upper[0]=pyclaw.BC.extrap
    solver.bc_lower[1]=pyclaw.BC.wall
    solver.bc_upper[1]=pyclaw.BC.extrap
    solver.aux_bc_lower[0]=pyclaw.BC.wall
    solver.aux_bc_upper[0]=pyclaw.BC.extrap
    solver.aux_bc_lower[1]=pyclaw.BC.wall
    solver.aux_bc_upper[1]=pyclaw.BC.extrap

    x = pyclaw.Dimension(-1.0,1.0,num_cells[0],name='x')
    y = pyclaw.Dimension(-1.0,1.0,num_cells[1],name='y')
    domain = pyclaw.Domain([x,y])

    num_eqn = 3
    num_aux = 2 # density, sound speed
    state = pyclaw.State(domain,num_eqn,num_aux)

    grid = state.grid
    X, Y = grid.p_centers

    rho_left   = 4.0  # Density in left half
    rho_right  = 1.0  # Density in right half
    bulk_left  = 4.0  # Bulk modulus in left half
    bulk_right = 4.0  # Bulk modulus in right half
    c_left = np.sqrt(bulk_left/rho_left)     # Sound speed (left)
    c_right = np.sqrt(bulk_right/rho_right)  # Sound speed (right)
    state.aux[0,:,:] = rho_left*(X<0.) + rho_right*(X>=0.) # Density
    state.aux[1,:,:] = c_left*(X<0.) + c_right*(X>=0.)     # Sound speed

    # Set initial condition
    x0 = -0.5; y0 = 0.
    r = np.sqrt((X-x0)**2 + (Y-y0)**2)
    width = 0.1; rad = 0.25
    state.q[0,:,:] = (np.abs(r-rad)<=width)*(1.+np.cos(np.pi*(r-rad)/width))
    state.q[1,:,:] = 0.
    state.q[2,:,:] = 0.

    claw = pyclaw.Controller()
    claw.keep_copy = True
    if disable_output:
        claw.output_format = None
    claw.solution = pyclaw.Solution(state,domain)
    claw.solver = solver
    claw.outdir = outdir
    claw.tfinal = 0.6
    claw.num_output_times = 20
    claw.write_aux_init = True
    claw.setplot = setplot
    if use_petsc:
        claw.output_options = {'format':'binary'}

    return claw


def setplot(plotdata):
    """ 
    Plot solution using VisClaw.

    This example shows how to mark an internal boundary on a 2D plot.
    """ 

    from clawpack.visclaw import colormaps

    plotdata.clearfigures()  # clear any old figures,axes,items data
    
    # Figure for pressure
    plotfigure = plotdata.new_plotfigure(name='Pressure', figno=0)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.title = 'Pressure'
    plotaxes.scaled = True      # so aspect ratio is 1
    plotaxes.afteraxes = mark_interface

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = 0
    plotitem.pcolor_cmap = colormaps.yellow_red_blue
    plotitem.add_colorbar = True
    plotitem.pcolor_cmin = 0.0
    plotitem.pcolor_cmax=1.0
    
    # Figure for x-velocity plot
    plotfigure = plotdata.new_plotfigure(name='x-Velocity', figno=1)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.title = 'u'
    plotaxes.afteraxes = mark_interface

    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = 1
    plotitem.pcolor_cmap = colormaps.yellow_red_blue
    plotitem.add_colorbar = True
    plotitem.pcolor_cmin = -0.3
    plotitem.pcolor_cmax=   0.3
    
    return plotdata

def mark_interface(current_data):
    import matplotlib.pyplot as plt
    plt.plot((0.,0.),(-1.,1.),'-k',linewidth=2)


if __name__=="__main__":
    from clawpack.pyclaw.util import run_app_from_main
    output = run_app_from_main(setup,setplot)



"""
Sample b4run.py file, defining a function b4run(rundir,outdir).  

If a file like this is in the rundir directory used by runclaw.py 
(e.g. the application directory from which you execute 'make .output'),
this will be executed after creating the outdir and before running 
the Clawpack code.

If no b4run.py file is found in the rundir then the environment variable B4RUN,
if set, is used to locate a file to be used more globally, e.g. you could
set this to point to $HOME/b4ruon.py for a default version to use if there
is no local version, or you can use this version globally by setting B4RUN to
$CLAW/clawutil/src/python/clawutil/b4run.py

Use this capability, for example, to copy any files you want to the outdir,
either because they are needed for the run or to archive them along
with the output.

This sample version copies the Makefile and any Python or Fortran codes.
Note that *.data files are automatically copied by runclaw.

It also creates (or appends to) a runlog.txt file containing information 
about this run: the rundir and the time/date the run started.

Adapt this file to your own needs.
"""


def b4run(rundir, outdir):

    import os,sys,glob,shutil,time

    # ---------------------------------------------------------------------
    # files to copy to outdir (in addition to *.data files always copied):
    to_copy = ['Makefile', '*.py', '*.f*']

    if rundir != outdir:
        for pattern in to_copy:
            files = glob.glob(pattern)  # files matching pattern
            for file in files:
                print('Copying %s to %s' % (file, outdir))
                shutil.copy(file, os.path.join(outdir,file))

    # ---------------------------------------------------------------------
    # also add to outdir/runlog.txt with info about when run was done
    # and from what directory:

    runlog = os.path.join(outdir, 'runlog.txt')

    tm = time.localtime()
    year = str(tm.tm_year).zfill(4)
    month = str(tm.tm_mon).zfill(2)
    day = str(tm.tm_mday).zfill(2)
    hour = str(tm.tm_hour).zfill(2)
    minute = str(tm.tm_min).zfill(2)
    tz = tm.tm_zone
    dt = '%s-%s-%s-%s:%s%s'  % (year,month,day,hour,minute,tz)

    with open(runlog,'a') as f:
        f.write('Run started at %s\n' % dt)
        f.write('from RUNDIR =\n   %s\n' % os.path.abspath(rundir))




#!/usr/bin/env python

def configuration(parent_package='',top_path=None):
    from numpy.distutils.misc_util import Configuration
    config = Configuration('sharpclaw', parent_package, top_path)

    config.add_extension('sharpclaw1',
                         ['ClawParams.f90','weno.f90','reconstruct.f90',
                          'evec.f90','workspace.f90','flux1.f90'])

    config.add_extension('sharpclaw2',
                         ['ClawParams.f90','weno.f90','reconstruct.f90',
                          'evec.f90','workspace.f90','flux2.f90',
                          'flux1.f90'])

    config.add_extension('sharpclaw3',
                         ['ClawParams.f90','weno.f90','reconstruct.f90',
                          'evec.f90','workspace.f90','flux3.f90',
                          'flux1.f90'])

    return config

if __name__ == '__main__':
    from numpy.distutils.core import setup
    setup(**configuration(top_path='').todict())



#!/usr/bin/env python
# encoding: utf-8

from __future__ import absolute_import
def burgers():
    """
    Example from Chapter 11 of LeVeque, Figure 11.8.
    Shows decay of an initial wave packet to an N-wave with Burgers' equation.
    """
    import numpy as np

    from clawpack import pyclaw
    from clawpack import riemann

    solver = pyclaw.ClawSolver1D(riemann.burgers_1D)

    solver.limiters = pyclaw.limiters.tvd.MC
    solver.bc_lower[0] = pyclaw.BC.periodic
    solver.bc_upper[0] = pyclaw.BC.periodic

    x = pyclaw.Dimension(-8.0,8.0,1000,name='x')
    domain = pyclaw.Domain(x)
    num_eqn = 1
    state = pyclaw.State(domain,num_eqn)

    xc = domain.grid.x.centers
    state.q[0,:] = (xc>-np.pi)*(xc<np.pi)*(2.*np.sin(3.*xc)+np.cos(2.*xc)+0.2)
    state.q[0,:] = state.q[0,:]*(np.cos(xc)+1.)
    state.problem_data['efix']=True

    claw = pyclaw.Controller()
    claw.tfinal = 6.0
    claw.num_output_times   = 30
    claw.solution = pyclaw.Solution(state,domain)
    claw.solver = solver

    return claw

if __name__=="__main__":
    from clawpack.pyclaw.util import run_app_from_main
    output = run_app_from_main(burgers)



#!/usr/bin/env python

"""Regression test for GeoClaw's storm surge functionality"""

from __future__ import absolute_import
import sys
import os
import unittest
import gzip
import nose

try:
    # For Python 3.0 and later
    from urllib.error import URLError
except ImportError:
    # Fall back to Python 2's urllib2
    from urllib2 import URLError

import numpy

import clawpack.geoclaw.test as test
import clawpack.geoclaw.topotools
from clawpack.geoclaw.surge import storm

class IkeTest(test.GeoClawRegressionTest):

    r"""Hurricane Ike regression test"""

    def setUp(self):

        super(IkeTest, self).setUp()

        # Download storm data
        remote_url = "http://ftp.nhc.noaa.gov/atcf/archive/2008/bal092008.dat.gz"
        try:
            path = self.get_remote_file(remote_url, unpack=False)
        except URLError:
            raise nose.SkipTest("Could not fetch remote file, skipping test.")
        
        storm_path = os.path.join(os.path.dirname(path), 'ike.storm')

        # Need to additionally deal with the fact the file is gzipped
        with gzip.GzipFile(path, 'r') as gzip_file:
            file_content = gzip_file.read()
        
        with open(storm_path+'.atcf', 'wb') as out_file:
            out_file.write(file_content)
            
        # now convert to geoclaw format
        ike_storm = storm.Storm(storm_path+'.atcf', file_format='ATCF', verbose=True)
        ike_storm.write(storm_path)
        
        # Download file
        #self.get_remote_file(
        #   "http://www.columbia.edu/~ktm2132/bathy/gulf_caribbean.tt3.tar.bz2")

        # Create synthetic bathymetry - needs more work
        topo = clawpack.geoclaw.topotools.Topography()
        topo.x = numpy.linspace(-100, -69, 125)
        topo.y = numpy.linspace(7.0, 33.0, 105)
        topo.Z = 25.0 * ((topo.X + 84.5)**2 + (topo.Y - 20.0)**2) - 4000.0
        topo.write(os.path.join(self.temp_path, 'gulf_caribbean.tt3'), \
                topo_type=2, Z_format="%22.15e")


    def runTest(self, save=False, indices=(2, 3)):
        r"""Storm Surge Regression Test

        :Input:
         - *save* (bool) - If *True* will save the output from this test to 
           the file *regresion_data.txt*.  Passed to *check_gauges*.  Default is
           *False*.
         - *indices* (tuple) - Contains indices to compare in the gague 
           comparison and passed to *check_gauges*.  Defaults to *(2, 3)*.

        """

        # Write out data files
        self.load_rundata()
        self.write_rundata_objects()

        # Run code
        self.run_code()

        # Perform tests
        self.check_gauges(save=save, gauge_id=1, indices=indices)

        # If we have gotten here then we do not need to copy the run results
        self.success = True


    # def check_gauges(self, save=False, indices=(2, 3)):
    #     r"""Basic test to assert gauge equality

    #     :Input:
    #      - *save* (bool) - If *True* will save the output from this test to 
    #        the file *regresion_data.txt*.  Default is *False*.
    #      - *indices* (tuple) - Contains indices to compare in the gague 
    #        comparison.  Defaults to *(2, 3)*.
    #     """

    #     # Get gauge data
    #     data = numpy.loadtxt(os.path.join(self.temp_path, 'fort.gauge'))
    #     data_sum = []
    #     for index in indices:
    #         data_sum.append(data[:, index].sum())

    #     # Get (and save) regression comparison data
    #     regression_data_file = os.path.join(self.test_path, "regression_data.txt")
    #     if save:
    #         numpy.savetxt(regression_data_file, data)
    #     regression_data = numpy.loadtxt(regression_data_file)
    #     regression_sum = []
    #     for index in indices:
    #         regression_sum.append(regression_data[:, index].sum())
    #     # regression_sum = regression_data

    #     # Compare data
    #     tolerance = 1e-14
    #     assert numpy.allclose(data_sum, regression_sum, tolerance), \
    #             "\n data: %s, \n expected: %s" % (data_sum, regression_sum)
    #     # assert numpy.allclose(data, regression_data, tolerance), \
    #     #         "Full gauge match failed."

if __name__=="__main__":
    if len(sys.argv) > 1:
        if bool(sys.argv[1]):
            # Fake the setup and save out output
            test = IkeTest()
            try:
                test.setUp()
                test.runTest(save=True)
            finally:
                test.tearDown()
            sys.exit(0)
    unittest.main()



#!/usr/bin/env python 

# convert CLAWPACK source code, data files, READMEs, etc. into html.
# for example,
#   clawcode2html filename.f
# generates
#   filename.f.html
# If filename.f.html already exists, you will be prompted before
# overwriting unless the -f or --force flag is used, e.g.
#   clawcode2html --force filename.f
#
# Based on mathcode2html.py from 
#    http://www.amath.washington.edu/~rjl/mathcode2html
# with some modifications for CLAWPACK.
#
# The environment variable CLAW must be properly set to the root
# of the claw directory before using this script.
# 
# Most of code is put into <pre> environment.
# Any comments enclosed by begin_html and end_html are taken out 
# of the <pre> environment and indented properly (using a table) 
# to match the surrounding code.

# By default, the html comments are offset in blue font, except if the 
# input file has extension .txt or no extension, in which case it's 
# left as black.  The default_color can be changed below.
# Also, the begin_html line may contain [color: red] for example,
# to determine the color of this comment.

# You may modify the recognized extensions and comment characters below.

# Allows many latex commands in comments if jsMath is used on webpage.
# In this case make sure the variable jsMathScript is properly set below.
# If you don't want to include a call to the jsMath script on the html
# page, invoke mathcode2html with the --nojsmath option.


#---------------------------------------------------------------------------
# Copyright (C) 2008   Randall J. LeVeque 
#
# Distributed as part of Clawpack, under the BSD license
# See www.clawpack.org
#---------------------------------------------------------------------------


import sys,os,glob
import re
import time
try:
    from optparse import OptionParser
except:
    print('You must use a more recent version of Python')
    sys.exit(1)

# parse command line arguments:
parser = OptionParser()
parser.add_option("-f", "--force",
                  action="store_true", dest="forced", default=False,
                  help="force action even if it overwrites html file")
parser.add_option("--nojsmath",
                  action="store_false", dest="nojsMath", default=False,
                  help="don't include call to jsMath script in html")
parser.add_option("-q", "--quiet",
                  action="store_false", dest="verbose", default=True,
                  help="don't print status messages to stdout")
parser.add_option("--noheader",
                  action="store_false", dest="header", default=True,
                  help="suppress printing header at top of html page")
parser.add_option("--dropext",
                  action="store_true", dest="dropext", default=False,
                  help="drop the file extension before adding .html for output")
parser.add_option("--eagle",
                  action="store_true", dest="eagle", default=False,
                  help="load the eagleclaw.css style")

(options, infiles) = parser.parse_args()

forced = options.forced
nojsMath = options.nojsMath
verbose = options.verbose
header = options.header
dropext = options.dropext 
eagle = options.eagle 



# CLAWPACK environment variables:

try:
    clawdir = os.getenv('CLAW')  # assumes environment variable set properly
except:
    print("CLAW environment variable not set")
    print("You need to run setenv.py before setup.py")
    sys.exit(1)

if clawdir == None:
    print("CLAW environment variable not set")
    print("You need to run setenv.py before setup.py")
    sys.exit(1)


# Create addresses for links on html pages.
# change the lines below if you want to point to a webpage home instead
# of the local file system  (e.g.  clawaddr = "http://www.mywebpage/claw")

#clawaddr = 'http://localhost:50005'

clawaddr = 'http://www.clawpack.org/'


# Set comment characters for different programming languages:
# Augment or modify as desired.
commentchar = { '.f'  : ['!', '#'], \
                '.f95'  : ['!','#'], \
                '.f90'  : ['!','#'], \
                '.m'  : '%', \
                '.py' : '#', \
                '.sh' : '#', \
                '.data' : ['#','=:'], \
                '.txt': None, \
                'Makefile': '#', \
                ''    : None}

firstfort = ['c','*','C','!']   # valid fortran .f comment char's in col. 1
firstfort95 = ['!']   # valid fortran .f95 comment char's in col. 1

leadingindent = ''    # additional indentation for webpage, if desired

default_color = 'blue'


try:
    infiles.remove('clawcode2html.py')  # can't apply this code to itself!
except:
    pass


for infilename in infiles:

    # check if this is a code file of a recognized language.
    ext = os.path.splitext(infilename)[1]
    if infilename == 'Makefile':
        ext = 'Makefile'  # special case
    
    if ext not in commentchar:
        print("  ")
        print("  Warning: Unrecognized extension, will proceed")
        print("           with no replacement of comment characters")
        commentchar[ext] = None 
    
    
    # open input and output files:
    #-----------------------------
    
    try:
        ifile = open(infilename,'r')
    except:
        print("File not found:", infilename)
        sys.exit(1)

    # Search for [use: ...] statements in file:
    all_lines = ifile.read()
    regexp = re.compile(r"\[use:[^\]]*jsMath")
    result = regexp.search(all_lines)

    # use jsMath if [use:jsMath] found and option --nojsmath was
    # not used in call:
    usejsMath = (result is not None) & (not nojsMath)

    regexp = re.compile(r"\[use:(?P<cssfile>[^\]]*).css\]")
    result = regexp.search(all_lines)
    if result is not None:
        cssfile = result.group('cssfile').strip() + '.css'
    else:
        cssfile = None

    
    ifile.seek(0)  # return to start of file
    lines = ifile.readlines()
    
    if dropext:
        infileroot = os.path.splitext(infilename)[0]
        outfilename = infileroot + '.html'
    else:
        outfilename = infilename + '.html'

    if (glob.glob(outfilename) != []) & (not forced):
        sys.stdout.write('  OK to overwrite %s?  '  %  outfilename)
        answer = input()
        if answer not in ['y','Y','yes']:
            print('  Aborting!')
            sys.exit(1)
    
    ofile = open(outfilename,'w')
    
    
    # start creating html file:
    #--------------------------
    
    if verbose:
        print('  Converting ', infilename, ' to ', outfilename)

    
    ofile.write("""<!-- **** DO NOT EDIT THIS FILE *** 
         This file was generated automatically from file
              %s
         using $CLAW/clawutil/src/python/clawutil/clawcode2html.py         -->""" % infilename)

    ofile.write('\n\n<html>\n<title> %s </title>\n\n'  % outfilename)
    if eagle:
        ofile.write("""
          <head>
          <link type="text/css" rel="stylesheet"
          href="%s/eagleclaw/eagleclaw.css">
          <link rel="icon" href="%s/_static/clawicon_new.ico" />
          </head> 
          <body>
        """ % (clawaddr,clawaddr))
    elif cssfile:
        ofile.write("""
          <head>
          <link type="text/css" rel="stylesheet"
          href="%s/%s">
          <link rel="icon" href="%s/_static/clawicon_new.ico" />
          </head> 
          <body>
        """ % (clawaddr, cssfile, clawaddr))
    else:
        ofile.write(
           """
          <head>
           <link rel="icon" href="%s/_static/clawicon_new.ico" />
          </head> 
           <BODY TEXT="#000000" BGCOLOR="#FFFFFF" LINK="#0000FF" 
                 VLINK="#5500DD" ALINK="#FF0000">
           <font FACE="HELVETICA,ARIAL">
           """ % clawaddr)

    
    # determine time and reformat:
    time1 = time.asctime()
    year = time1[-5:]
    day = time1[:-14]
    hour = time1[-13:-5]
    creationtime = day + year + ' at ' + hour

    # put full file name with path into a comment for future reference:
    fullinfilename = os.path.join(os.getcwd(),infilename)
    ofile.write('\n<!-- Created from the file %s -->\n'  % fullinfilename)
    ofile.write('<!-- Date: %s -->\n\n'  % creationtime)
    
    
    if header:
        #ofile.write('<table bgcolor="#DDEEEE"> <tr> <td>\n')
        ofile.write('<table bgcolor="#FFEE99"> <tr> <td>\n')
        ofile.write('&nbsp;<font size=6> %s </font> </td>\n' % outfilename)
    
        ofile.write("""
            <td><a href="http://www.clawpack.org/index.html"><img
            src="%s/_static/clawlogo_border.jpg"
            width=150 alt="CLAWPACK"></a>""" % clawaddr)

        ofile.write('&nbsp;&nbsp; </td> </tr> <tr> <td>\n')
    
        ofile.write('&nbsp;Source file: &nbsp;&nbsp;<a href="%s">%s </a>\n' \
                     % (infilename,infilename))
        ofile.write('</td></tr><tr><td>\n')
        ofile.write('&nbsp;Directory:  &nbsp; %s \n' %  os.getcwd())
        ofile.write('</td></tr><tr><td>\n')
        ofile.write('&nbsp;Converted:  &nbsp; %s \n' % creationtime)
        ofile.write('&nbsp; using <a href="%s/doc/application_documentation.html#clawcode2html">clawcode2html</a>\n'\
                    % clawaddr)
        ofile.write('</td></tr><tr><td>\n')
        ofile.write('<font color="#BB3300"> &nbsp;This documentation file will \n')
        ofile.write('not reflect any later changes in the source file. </font>\n')
        ofile.write('</td></tr></table></font>\n')
        ofile.write('<p>\n')
    
    
    if usejsMath:
        
        # Set jsMathScript to the file or URL of the java script load.js.

        # proper location for using webserver started by executing
        # "python startserver.py" in the $CLAW directory:
        jsMathScript = "%s/doc/load.js"  % clawaddr

        # script location for posting on kingkong server:
        # jsMathScript = "http://kingkong.amath.washington.edu/claw/doc/load.js"
    
        # script location for posting on rjl's webpage:
        # jsMathScript = "http://www.amath.washington.edu/~rjl/jsMath/easy/load.js"
    
        ofile.write("    <!-- jsMath stuff:  -->\n")
        ofile.write(" <SCRIPT SRC= %s> </SCRIPT>\n"  % jsMathScript)

    
        ofile.write("""
            <!-- Warning message if script doesn't work:  -->
            $\phantom{******** If you see this on the webpage then the 
                               browser could not locate *********}$<br>
            $\phantom{******** the jsMath file load.js *********}$<p>
            \n""")
        
        # define any latex macros that you want to use in jsMath: 
        
        ofile.write("""
          <!- latex macros: -->
          $\\newcommand{\\vector}[1]{\\left[\\begin{array}{c} #1 \\end{array}\\right]}$ 
          $\\newenvironment{matrix}{\\left[\\begin{array}{cccccccccc}} {\\end{array}\\right]}$ 
          $\\newcommand{\\A}{{\\cal A}}$
          $\\newcommand{\\W}{{\\cal W}}$
          \n""")
        #
        # end of jsMath stuff
    

    # start writing input file to output file...
    
    # code is in pre-formatted environment:
    ofile.write('<pre> \n')
    
    insidehtml = 0;   # set to 1 when we're processing html comments
    lineno = 0;       # line number counter for error message
    
    for line in lines:
    
        lineno += 1
    
        if 'begin_html' in line:
            regexp = re.compile(r"\[color:(?P<color>[^\]]*)\]")
            result = regexp.search(line)
            if result:
                font_color = result.group('color')
            else:
                font_color = default_color
    
            if insidehtml:
                print('  Error at line ', lineno, '\n')
                print('  Unexpected begin_html  when already in html mode\n')
                print('  Missing end_html? \n')
                sys.exit(1)
    
            # switch out of pre-formatted mode and create table to indent
            ofile.write('</pre>\n<table><tr><td>\n')
    
            # The first column of the table is spaces for indentation 
            # to match surrounding source code.
            # Count how many spaces there are before the begin_html
            #  or to the first comment character preceeding that string:
            regexp = re.compile(r"(?P<spaces>[ ]*)(#|%|begin)")
            result = regexp.search(line)
            if result:
                numindent = len(leadingindent) + len(result.group('spaces'))
                numindent = numindent
                ofile.write('<pre><tt>')
                for i in range(numindent):
                    ofile.write(' ')
                ofile.write('</tt></pre>')
            else:
                print('  Strange error in clawcode2html - should not be here')
                print('     at line number ', lineno)
              
    
            ofile.write('\n</td>\n')
    
            # the next column of the table has the comment itself:
            ofile.write('<td width=900>\n')
            if ext not in ('.txt', ''):
                # use colored font for comments except for text files.
                ofile.write('<font size="-1" color=%s>\n'  % font_color)
            insidehtml = 1;
    
        elif 'end_html' in line:
            # switch back to pre-formatted environment
            ofile.write('</font></td></tr></table>\n')
            ofile.write('<pre> \n')
            insidehtml = 0;
    
        else:
            if insidehtml:
    
                # replace blank line in html comment by new paragraph <p>:
                blankline = (line.strip == '')
                if not blankline:
                   firstchar = line.split()[0][0]
                   if ((ext in ['.f','.f95']) & (firstchar not in firstfort)): 
                       if firstchar not in commentchar[ext]:
                           print('  Error... in line ', lineno,'\n' \
                                 '  In html but not in a comment.'\
                                 '   Forgotten "end_html" ?')
                           sys.exit(1)
    
                # fortran comments may have comment symbol in column 1
                # strip lines inside an html comment of leading symbol:
                if ext == '.f':
                    if line[0] in firstfort:
                       line = ' ' + line[1:]
                if ext == '.f95':
                    if line[0] in firstfort95:
                       line = ' ' + line[1:]
    
                # replace any comment character by ' ' 
                if commentchar[ext]:
                    for char in commentchar[ext]:
                        line = line.replace(char,' ')  
    
    
                blankline = (line.strip == '')
                if blankline:
                   line = ('<p>\n')


                # Allow wiki formatting of links:
                # -------------------------------

                # Replace [name: placemark] by <a name="placemark">
                # (to jump to a different spot in the same html file)
                regexp = re.compile(r"\[name:[ ]*(?P<placemark>[^ ^\]]*)\]")
                result = regexp.search(line)
                while result:
                    placemark = result.group('placemark')
                    oldpat = result.group()
                    newpat = '<a name="%s">' % placemark
                    line = line.replace(oldpat,newpat)
                    result = regexp.search(line)

                # Replace links of the form [code: target]
                # by html links to both target and target.html.
                # Also allows [code: target#placemark] with links to target 
                # and target.html#placemark.

                regexp = re.compile(r"\[code:[ ]*(?P<target>[^ ^\]^#]*)([#]?)" + \
                                    r"(?P<placemark>[^\]]*)\]")
                result = regexp.search(line)
                while result:
                    targetname = result.group('target')
                    placemark = result.group('placemark')
                    oldpat = result.group()
                    if placemark:
                        newpat = '<a href="%s">%s</a>' \
                                     % (targetname,targetname) + \
                                 '&nbsp;<a href="%s.html#%s">[.html]</a>' \
                                     % (targetname,placemark) 
                    else:
                        oldpat = result.group()
                        newpat = '<a href="%s">%s</a>' \
                                     % (targetname,targetname) + \
                                 '&nbsp;<a href="%s.html">[.html]</a>' \
                                     % targetname 
                    line = line.replace(oldpat,newpat)
                    result = regexp.search(line)


                # replace links of the form [link: target text]
                # by an html link from text to the target page.
                regexp = re.compile(r"\[link:[ ]?(?P<target>[^ ^\]]*)" + \
                                    r"([ ]*)(?P<text>[^\]]*)\]")
                result = regexp.search(line)
                while result:
                    targetname = result.group('target')
                    text = result.group('text')
                    oldpat = result.group()
                    if text=='': 
                        text = targetname
                    newpat = '<a href="' + targetname + '">' + \
                                text + '</a>  '
                    line = line.replace(oldpat,newpat)
                    result = regexp.search(line)


                # replace links of the form [http:etc text]
                # by an html link from text to the http page.
                regexp = re.compile(r"\[http:(?P<target>[^ ^\]]*)" + \
                                    r"([ ]*)(?P<text>[^\]]*)\]")
                result = regexp.search(line)
                while result:
                    targetname = result.group('target')
                    text = result.group('text')
                    oldpat = result.group()
                    if text=='': 
                        text = targetname[2:]
                    newpat = '<a href="http:' + targetname + '">' + \
                                text + '</a>  '
                    line = line.replace(oldpat,newpat)
                    result = regexp.search(line)


                # replace links of the form [www.etc text]
                # by an html link from text to the http://www.etc page.
                regexp = re.compile(r"\[www.(?P<target>[^ ^\]]*)" + \
                                    r"([ ]*)(?P<text>[^\]]*)\]")
                result = regexp.search(line)
                while result:
                    targetname = result.group('target')
                    text = result.group('text')
                    oldpat = result.group()
                    if text=='': 
                        text = 'www.' + targetname
                    newpat = '<a href="http://www.' + targetname + '">' + \
                                text + '</a>  '
                    line = line.replace(oldpat,newpat)
                    result = regexp.search(line)


                # special things for CLAWPACK:

                # replace links of the form [clawcode:clawpack/1d/lib/step1.f]
                # for example by links relative to clawaddr, 
                # along with a link to the .html version.
                regexp = re.compile(r"\[clawcode:[ ]*(?P<target>[^ ^\]^#]*)([#]?)" + \
                                    r"(?P<placemark>[^\]]*)\]")
                result = regexp.search(line)
                while result:
                    targetname = result.group('target').lstrip()
                    placemark = result.group('placemark')
                    oldpat = result.group()
                    if placemark:
                        newpat = '<a href="%s/%s">claw/%s</a>' \
                                     % (clawaddr,targetname,targetname) + \
                                 '&nbsp;<a href="%s/%s.html#%s">[.html]</a>' \
                                     % (clawaddr,targetname,placemark) 
                    else:
                        newpat = '<a href="%s/%s">%s</a>' \
                                     % (clawaddr,targetname,targetname) + \
                                 '&nbsp;<a href="%s/%s.html">[.html]</a>' \
                                     % (clawaddr,targetname) 
                    line = line.replace(oldpat,newpat)
                    result = regexp.search(line)


                # replace links of the form [claw:clawpack/1d/lib]
                # for example by links relative to clawaddr,
                # with no .html version.
                regexp = re.compile(r"\[claw:[ ]?(?P<target>[^ ^\]]*)" + \
                                    r"([ ]*)(?P<text>[^\]]*)\]")
                result = regexp.search(line)
                while result:
                    targetname = result.group('target')
                    text = result.group('text')
                    oldpat = result.group()
                    if text=='': 
                        text = '$CLAW/' + targetname
                    newpat = '<a href="' + clawaddr + '/' + targetname + \
                               '">' + text + '</a>  ' 
                    line = line.replace(oldpat,newpat)
                    result = regexp.search(line)

                # place text surrounded by triple braces with 
                # pre environment with background color:
                newpat = '<pre class="clawcode">'
                line = line.replace('{{{',newpat)
                line = line.replace('}}}','</pre>')

          
            else:
                # not insidehtml - make regular comments default_color.
                # Determine if this line contains a comment and if so,
                # what column the comment starts in:

                startcomment = 1000
                if (ext == '.f') & (line[0] in firstfort):
                    startcomment = 0
                elif (ext == '.f95') & (line[0] in firstfort95):
                    startcomment = 0
                else:
                    if commentchar[ext]:
                        for c in commentchar[ext]:
                            commentcol = line.find(c)
                            if (commentcol>-1)&(commentcol<startcomment):
                                startcomment = commentcol

                if startcomment<1000:
                    line = line[0:startcomment] + \
                           '<font color="%s">'  % default_color + \
                           line[startcomment:-1] + '</font>\n'


            # output the (possibly modified) line to the output file:
            ofile.write('%s' % leadingindent+line)
    
    # Done with all lines.  Add closing stuff at bottom of html file:
    ofile.write('</pre></html>\n')
    
    ifile.close()
    ofile.close()
    



#!/usr/bin/env python
__doc__ = r"""
Module to diff two images or directories full of images.

Can by used from the command line via:
   python imagediff.py file1 file2
   python imagediff.py dir1 dir2 [dir3]
In the latter case dir1 and dir2 are any two directories and all
images will be compared.  The optional dir3 argument gives the directory
for the resulting diff's and html files.  Default is '_image_diff'.

Command line flags include:
    -v, --verbose = Verbose output   
    -r, --relocatable = Copy original image files to dir3 also
    -h, --help = Display this help
"""

help_message = __doc__

import os, sys, getopt, shutil

def imagediff_file(fname1, fname2, verbose=True):
    
    fname3 = make_imagediff(fname1,fname2,verbose=verbose)
    
    hfile = "imagediff.html"
    html = open(hfile,"w")
    html.write("""
         <html>
         <h2>Comparison of image files</h2>
         Left: %s <br>
         Right: %s
         <p>
         <img src="%s" width=400 border="1"> &nbsp; <img src="%s" width=400 border="1">""" \
      % (fname1,fname2,fname1,fname2))
      
    html.write("""
         <h2>Pixel difference is nonzero where image below is black...</h2>
         <img src="%s" width=400 border="1">""" % fname3)
    html.close()
    print("To view comparision and pixelwise difference, see ", hfile)
    
    
         
def make_imagediff(fname1,fname2,fname3='', verbose=False):
    ext1 = os.path.splitext(fname1)[1]
    ext2 = os.path.splitext(fname2)[1]
    if ext1 != ext2:
        print("*** Error: extensions of fname1 and fname2 must agree")
        return
    if ext1 not in ['.png','.gif','.jpg']:
        print("*** Error: image files not recognized")
        return
    
    if fname3 == '':
        fname3 = "_image_diff" + ext1
    
    os.system("convert %s %s -compose subtract -composite -black-threshold 0.01 -negate %s" \
         % (fname1, fname2, fname3))
    
    if verbose:     
        print("Created pixelwise difference ", fname3)
    return fname3
    
    
def imagediff_dir(dir1, dir2, dir3="_image_diff", ext='.png', \
                  regression_test_files='all',  \
                  relocatable=False, overwrite=False, verbose=False):
    
    import filecmp,glob
    from numpy import alltrue
    
    if dir1[-1] == '/': dir1 = dir1[:-1]
    if dir2[-1] == '/': dir2 = dir2[:-1]
    dir1 = os.path.abspath(dir1)
    dir2 = os.path.abspath(dir2)
    
    
    files1 = glob.glob("%s/*%s" % (dir1,ext))    
    files1 = [f.replace(dir1+'/','') for f in files1]
    files2 = glob.glob("%s/*%s" % (dir2,ext))
    files2 = [f.replace(dir2+'/','') for f in files2]
    
    files_both = files1 + files2
    files = []
    for f in files_both:
        if f not in files: files.append(f)
    files.sort()
    
    print("Comparing files in the  directory: ", dir1)
    print("               with the directory: ", dir2)

    if os.path.isdir(dir3):
        if (len(os.listdir(dir3)) > 0)  and (not overwrite):
            ans = input("Ok to overwrite files in %s ?  " % dir3)
            if ans.lower() not in ['y','yes']:
                print("*** Aborting")
                return
    else:
        os.system('mkdir -p %s' % dir3)
    startdir = os.getcwd()


    
    
    os.chdir(dir3)
            
    hname = '_ImageDiffIndex.html'
    hfile = open(hname,'w')
    hfile.write("""<html>
            <h1>Figure comparison and pixel diff</h1>
            Comparing files in the directory: &nbsp; dir1 = %s<br>
            &nbsp;&nbsp; with the directory: &nbsp; dir2 = %s<br>
            &nbsp;&nbsp; with the image extension: %s<p>
            &nbsp;<p>
            <h2>Files:</h2>
            """ % (dir1,dir2,ext))
            
    f_equal, f_diff, f_other = filecmp.cmpfiles(dir1,dir2,files,False)

    if relocatable:
        # copy files from dir1 and dir2 into dir3 so the whole thing can
        # be moved elsewhere, e.g. to post on web for discussion of a bug.
        dir1copy = 'dir1'
        dir2copy = 'dir2'
        os.system('mkdir %s' % dir1copy)
        os.system('mkdir %s' % dir2copy)
        for f in files1:
            shutil.copy(os.path.join(dir1,f),dir1copy)
            ## Specific to Clawpack _plots, also want html file:
            fhtml = os.path.splitext(f)[0] + '.html'  
            if os.path.isfile(os.path.join(dir1,fhtml)): 
                shutil.copy(os.path.join(dir1,fhtml),dir1copy)

        for f in files2:
            shutil.copy(os.path.join(dir2,f),dir2copy)
            ## Specific to Clawpack _plots, also want html file:
            fhtml = os.path.splitext(f)[0] + '.html'  
            if os.path.isfile(os.path.join(dir2,fhtml)): 
                shutil.copy(os.path.join(dir2,fhtml),dir2copy)

        dir1 = 'dir1'
        dir2 = 'dir2'

    files_missing1 = []
    files_missing2 = []
    files_differ = []
    
    for f in files:
        fhtml = os.path.splitext(f)[0] + '.html'  ## Specific to Clawpack _plots
        if not os.path.isfile(os.path.join(dir1,fhtml)): fhtml = f
          
        fname1 = os.path.join(dir1,f)
        fname2 = os.path.join(dir2,f)
        fhtml1 = os.path.join(dir1,fhtml)
        fhtml2 = os.path.join(dir2,fhtml)

        if f not in files2:
            files_missing2.append(f)
            hfile.write("""
              <table>
              <tr><td><b>%s</b> only appears in dir1</td></tr>
              <tr>
              <td><a href="%s"><img src="%s" width=350 border="1"></a></td> &nbsp; &nbsp;
              <td><img src="XXmissingXX" width=350 height=250 border="1"></td>
              </tr>
              </table><p>"""  % (f, fhtml1, fname1))
        elif f not in files1:
            files_missing1.append(f)
            hfile.write("""
              <table>
              <tr><td></td><td><b>%s</b> only appears in dir2</td></tr>
              <tr>
              <td><img src="XXmissingXX" width=350 height=250 border="1"></td>&nbsp;&nbsp;
              <td><a href="%s"><img src="%s" width=350 border="1"></a></td> &nbsp; 
              </tr>
              </table><p>"""  % (f, fhtml2, fname2))
            
        elif f in f_equal:
            fname1 = os.path.join(dir1,f)
            hfile.write("""
              <table>
              <tr><td><b>%s</b> are identical in dir1 and dir2</td></tr>
              <tr>
              <td><a href="%s"><img src="%s" width=350 border="1"></a></td> &nbsp;&nbsp; 
              </tr>
              </table><p>"""  % (f, fhtml1, fname1))
            # hfile.write("""
            #               <a href="%s"><img src="%s" width=350 border="1"></a>""" \
            #                 % (fname1,fname1))
        else:
            files_differ.append(f)
            fname3 = f

            fname3 = make_imagediff(fname1,fname2,fname3,verbose=verbose)
            hfile.write("""
              <table>
              <tr><td><b>%s</b> from dir1</td><td><b>%s</b>  from dir2</td>
                  <td>Pixels that differ between the images</td>  </tr>
              <tr>
              <td><a href="%s"><img src="%s" width=350 border="1"></a></td> &nbsp;&nbsp; 
              <td><a href="%s"><img src="%s" width=350 border="1"></a></td> &nbsp;&nbsp; 
              <td><a href="%s"><img src="%s" width=350 border="1"></a></td>  </tr>
              </table><p>""" \
                % (f,f,fhtml1,fname1,fhtml2,fname2,fname3,fname3))

    # Test regression files for return value:
    if regression_test_files=='all':
        regression_test_files = files_both
    regression_ok = alltrue([f in f_equal for f in \
                                regression_test_files])
    if verbose and regression_ok:
        print("Regression files all match")
    elif verbose:
        print("*** Regression files are not all identical")
        print("*** Files missing from dir1:   ",files_missing1)
        print("*** Files missing from dir2:   ",files_missing2)
        print("*** Files that differ:   ",files_differ)
        
    os.chdir(startdir)
    dir3 = os.path.abspath(dir3)
    print("To view diffs, open the file ",os.path.join(dir3,hname))

    return regression_ok
    

class Usage(Exception):
    def __init__(self, msg):
        self.msg = msg


if __name__ == "__main__":    
    # Parse input arguments
    argv = sys.argv
    try:
        try:
            opts, args = getopt.getopt(argv[1:], "hv",["help","verbose","relocatable"])
        except getopt.error as msg:
            raise Usage(msg)

        # Default script parameter values
        verbose = False
        relocatable = False

        # option processing
        for option, value in opts:
            # Script parameters
            if option in ("-v","--verbose"):
                 verbose = True
            if option in ("-r","--relocatable"):
                 relocatable = True
            if option in ("-h","--help"):
                raise Usage(help_message)

        # Run diff
        if os.path.isfile(args[0]) and os.path.isfile(args[1]):
            sys.exit(imagediff_file(args[0],args[1],verbose=verbose))
        elif os.path.isdir(args[0]) and os.path.isdir(args[1]):
            if len(args)==3:
                sys.exit(imagediff_dir(args[0],args[1],args[2],verbose=verbose,\
                     relocatable=relocatable))
            else:
                sys.exit(imagediff_dir(args[0],args[1],verbose=verbose,\
                     relocatable=relocatable))
        else:
              raise Usage("Both paths must either be files or directories.")
        
    except Usage as err:
        print(sys.argv[0].split("/")[-1] + ": " + str(err.msg), file=sys.stderr)
        print("\t for help use --help", file=sys.stderr)
        sys.exit(2)
    
    
        
        
        



#!/usr/bin/env python

"""

Classes representing parameters for GeoClaw runs

:Classes:

 - GeoClawData
 - RefinementData
 - TopographyData
 - FGoutData
 - FGmaxData
 - DTopoData
 - QinitData
 - SurgeData
 - MultilayerData
 - FrictionData 
 - BoussData
 - GridData1D
 - BoussData1D

:Constants:

 - Rearth - Radius of earth in meters
 - DEG2RAD factor to convert degrees to radians
 - RAD2DEG factor to convert radians to degrees
 - LAT2METER factor to convert degrees in latitude to meters
"""

from __future__ import absolute_import
from __future__ import print_function
import os
import numpy
import clawpack.clawutil.data
import warnings


# Radius of earth in meters.
# For consistency, should always use this value when needed, e.g.
# in setrun.py or topotools:
Rearth = 6367.5e3  # average of polar and equatorial radii

DEG2RAD = numpy.pi / 180.0
RAD2DEG = 180.0 / numpy.pi
LAT2METER = Rearth * DEG2RAD

class GeoClawData(clawpack.clawutil.data.ClawData):
    r"""
    Object containing the basic .

    Note that this data object will write out multiple files.
    """
    def __init__(self):
        super(GeoClawData,self).__init__()

        # GeoClaw physics parameters
        self.add_attribute('gravity',9.8)
        self.add_attribute('rho', 1025.0)  # Density of water kg/m^3
        self.add_attribute('rho_air',1.15) # Density of air kg/m^3
        self.add_attribute('ambient_pressure', 101.3e3) # Nominal atmos pressure
        self.add_attribute('earth_radius',Rearth)
        self.add_attribute('coordinate_system',1)
        self.add_attribute('sphere_source',1)  # New starting in v5.10.0
        self.add_attribute('coriolis_forcing',True)
        self.add_attribute('theta_0',45.0)
        self.add_attribute('friction_forcing',True)
        self.add_attribute('manning_coefficient',[0.025])
        self.add_attribute('manning_break',[])

        # GeoClaw algorithm parameters
        self.add_attribute('dry_tolerance',1e-3)
        self.add_attribute('friction_depth',1.0e6)
        self.add_attribute('sea_level',0.0)


    def write(self,data_source='setrun.py', out_file='geoclaw.data'):

        self.open_data_file(out_file, data_source)

        self.data_write('gravity',
                               description="(gravitational acceleration m/s^2)")
        self.data_write('rho', description="(Density of water kg/m^3)")
        self.data_write('rho_air',description="(Density of air kg/m^3)")
        self.data_write('ambient_pressure',
                                description="(Nominal atmospheric pressure Pa)")
        self.data_write('earth_radius', description="(Radius of the earth m)")
        self.data_write('coordinate_system',
                        description="(1=meters, 2=lon-lat)")
        self.data_write('sphere_source',
                        description="(0=none, 1=only in mass eqn, 2=all)")
        self.data_write('sea_level')
        self.data_write()

        # Forcing terms
        self.data_write('coriolis_forcing')
        if self.coordinate_system == 1 and self.coriolis_forcing:
            self.data_write('theta_0')
        self.data_write('friction_forcing')
        if self.friction_forcing:
            if type(self.manning_coefficient) in [int,float]:
                self.manning_coefficient = [self.manning_coefficient]
            num_manning = len(self.manning_coefficient)
            if len(self.manning_break) != num_manning - 1:
                raise IOError("***manning_break array has wrong length")
            self.data_write(value=num_manning,alt_name='num_manning')
            self.data_write('manning_coefficient')
            self.data_write('manning_break')
            self.data_write('friction_depth')

        self.data_write()

        self.data_write('dry_tolerance')

        self.close_data_file()



class RefinementData(clawpack.clawutil.data.ClawData):

    def __init__(self):

        super(RefinementData,self).__init__()

        # Refinement controls
        self.add_attribute('wave_tolerance',1.0e-1)
        self.add_attribute('speed_tolerance',[1.0e12]*6)
        self.add_attribute('deep_depth',None)      # deprecated
        self.add_attribute('max_level_deep',None)  # deprecated
        self.add_attribute('variable_dt_refinement_ratios',False)


    def write(self,data_source='setrun.py', out_file='refinement.data'):
        # Refinement controls
        self.open_data_file(out_file, data_source)
        self.data_write('wave_tolerance')

        # check if user set deprecated parameters:
        if self.deep_depth is not None:
            w = '\n  *** WARNING: deep_depth parameter ignored as of v5.8.0'
            warnings.warn(w, UserWarning)
        if self.max_level_deep is not None:
            w = '\n  *** WARNING: max_level_deep parameter ignored as of v5.8.0'
            warnings.warn(w, UserWarning)

        if not isinstance(self.speed_tolerance,list):
            self.speed_tolerance = [self.speed_tolerance]
        self.data_write('speed_tolerance')
        self.data_write()
        self.data_write('variable_dt_refinement_ratios',
                        description="(Set dt refinement ratios automatically)")
        self.close_data_file()



class TopographyData(clawpack.clawutil.data.ClawData):

    def __init__(self):

        super(TopographyData,self).__init__()

        # Topography data
        self.add_attribute('topo_missing',99999.)
        self.add_attribute('test_topography',0)
        self.add_attribute('topofiles',[])

        # Jump discontinuity
        self.add_attribute('topo_location',-50e3)
        self.add_attribute('topo_left',-4000.0)
        self.add_attribute('topo_right',-200.0)
        self.add_attribute('topo_angle',0.0)

        # Simple oceanic shelf
        self.add_attribute('x0',350e3)
        self.add_attribute('x1',450e3)
        self.add_attribute('x2',480e3)
        self.add_attribute('basin_depth',-3000.0)
        self.add_attribute('shelf_depth',-100.0)
        self.add_attribute('beach_slope',0.008)


    def write(self,data_source='setrun.py', out_file='topo.data'):

        self.open_data_file(out_file, data_source)
        self.data_write(name='topo_missing',
                        description='replace no_data_value in topofile')
        self.data_write(name='test_topography',description='(Type topography specification)')
        if self.test_topography == 0:
            ntopofiles = len(self.topofiles)
            self.data_write(value=ntopofiles,alt_name='ntopofiles')
            for tfile in self.topofiles:

                if len(tfile) == 6:
                    w = '\n  *** WARNING: topofile specs changed in v5.8.0 -- ' + \
                          'Flag level info now ignored'
                    warnings.warn(w, UserWarning)
                    tfile = [tfile[0], tfile[-1]] # drop minlevel,maxlevel,t1,t2
                elif len(tfile) == 2:
                    pass  # now expect only topo_type, filename
                else:
                    raise ValueError('Unexpected len(tfile) = %i' % len(tfile))

                # if path is relative in setrun, assume it's relative to the
                # same directory that out_file comes from
                fname = os.path.abspath(os.path.join(os.path.dirname(out_file),tfile[-1]))
                self._out_file.write("\n'%s' \n " % fname)
                self._out_file.write("%3i   # topo_type\n" % tfile[0])
        elif self.test_topography == 1:
            self.data_write(name='topo_location',description='(Bathymetry jump location)')
            self.data_write(name='topo_left',description='(Depth to left of bathy_location)')
            self.data_write(name='topo_right',description='(Depth to right of bathy_location)')
        elif self.test_topography == 2 or self.test_topography == 3:
            self.data_write(name='x0',description='(Location of basin end)')
            self.data_write(name='x1',description='(Location of shelf slope end)')
            self.data_write(name='x2',description='(Location of beach slope)')
            self.data_write(name='basin_depth',description='(Depth of basin)')
            self.data_write(name='shelf_depth',description='(Depth of shelf)')
            self.data_write(name='beach_slope',description='(Slope of beach)')
        else:
            raise NotImplementedError("Test topography type %s has not been"
                                        " implemented." % self.test_topography)

        self.close_data_file()


class FixedGridData(clawpack.clawutil.data.ClawData):

    """
    Deprecated, starting in 5.9.0 use FGoutData instead.
    """

    def __init__(self):

        super(FixedGridData,self).__init__()

        # Fixed Grids
        self.add_attribute('fixedgrids',[])


    def write(self,data_source='setrun.py', out_file='fixed_grids.data'):
        # Fixed grid settings
        msg = 'rundata.fixed_grid_data is deprecated starting in v5.9.0,' \
            + ' use rundata.fgout_data instead'
        #warnings.warn(msg)
        if len(self.fixedgrids) > 0:
            raise AttributeError(msg)


class FGoutData(clawpack.clawutil.data.ClawData):

    def __init__(self):

        super(FGoutData,self).__init__()

        # File name for fgout points and parameters:
        self.add_attribute('fgout_grids',[])


    def write(self,data_source='setrun.py', out_file='fgout_grids.data'):
        self.open_data_file(out_file, data_source)
        num_fgout_grids = len(self.fgout_grids)
        self.data_write(value=num_fgout_grids,alt_name='num_fgout_grids')
        self.data_write()

        fgno_unset = 0  # to use if fg.fgno not set by user
        fgno_list = []  # to check for uniqueness of fgno's

        for fg in self.fgout_grids:
            # if path is relative in setrun, assume it's relative to the
            # same directory that out_file comes from

            if fg.fgno is None:
                # not set by user in setrun
                fgno_unset += 1
                fg.fgno = fgno_unset

            if fg.fgno in fgno_list:
                msg = 'Trying to set fgout grid number to fgno = %i' % fg.fgno \
                      + '\n             but this fgno was already used' \
                      + '\n             Set unique fgno for each fgout grid'
                raise ValueError(msg)

            fgno_list.append(fg.fgno)
            fg.write_to_fgout_data(self._out_file)
        self.close_data_file()

class FGmaxData(clawpack.clawutil.data.ClawData):

    def __init__(self):

        super(FGmaxData,self).__init__()

        # File name for fgmax points and parameters:
        self.add_attribute('fgmax_files',[])
        self.add_attribute('num_fgmax_val',1)
        self.add_attribute('fgmax_grids',[])


    def write(self,data_source='setrun.py', out_file='fgmax_grids.data'):
        if len(self.fgmax_files) > 0:
            msg = '*** fgmax_files has been deprecated, ' \
                  + 'use fgmax_grids instead.'
            raise ValueError(msg)

        # new style:
        self.open_data_file(out_file, data_source)
        num_fgmax_val = self.num_fgmax_val
        if num_fgmax_val not in [1,2,5]:
            raise NotImplementedError(
                   "Expecting num_fgmax_val in [1,2,5], got %s" % num_fgmax_val)
        self.data_write(value=num_fgmax_val, alt_name='num_fgmax_val')
        num_fgmax_grids = len(self.fgmax_grids)
        self.data_write(value=num_fgmax_grids, alt_name='num_fgmax_grids')
        self.data_write()

        fgno_unset = 0  # to use if fg.fgno not set by user
        fgno_list = []  # to check for uniqueness of fgno's

        for fg in self.fgmax_grids:
            # if path is relative in setrun, assume it's relative to the
            # same directory that out_file comes from
            if fg.xy_fname is not None:
                fg.xy_fname = os.path.abspath(os.path.join(\
                              os.path.dirname(out_file),fg.xy_fname))

            if fg.fgno is None:
                # not set by user in setrun
                fgno_unset += 1
                fg.fgno = fgno_unset

            if fg.fgno in fgno_list:
                msg = 'Trying to set fgmax grid number to fgno = %i' % fg.fgno \
                      + '\n             but this fgno was already used' \
                      + '\n             Set unique fgno for each fgmax grid'
                raise ValueError(msg)

            fgno_list.append(fg.fgno)
            fg.write_to_fgmax_data(self._out_file)
        self.close_data_file()


    def read(self, path="fgmax_grids.data", force=False):
        r"""Read a FGMax data file."""

        super(FGmaxData, self).read(path, force=force)

        # Look for basic parameters
        fig_numbers = []
        with open(os.path.abspath(path), 'r') as data_file:
            # Forward to first parameter
            for line in data_file:
                # Regular parameter setting
                if "=:" in line:
                    value, tail = line.split("=:")
                    varname = tail.split()[0]

                    if varname == "num_fgmax_val":
                        self.num_fgmax_val = int(value)
                    elif varname == "num_fgmax_grids":
                        num_fgmax_grids = int(value)
                
                # Contains a fixed grid number
                elif "# fgno" in line:
                    value, tail = line.split("#")
                    fig_numbers.append(int(value))

        if len(fig_numbers) != num_fgmax_grids:
            raise ValueError("Number of FGMaxGrid numbers found does not ", 
                             "equal the number of grids recorded.")
        
        # Read each fgmax grid
        import clawpack.geoclaw.fgmax_tools
        for (i, grid_num) in enumerate(fig_numbers):
            new_fgmax_grid = clawpack.geoclaw.fgmax_tools.FGmaxGrid()
            new_fgmax_grid.read_fgmax_grids_data(grid_num, data_file=path)
            self.fgmax_grids.append(new_fgmax_grid)



class DTopoData(clawpack.clawutil.data.ClawData):

    def __init__(self):

        super(DTopoData,self).__init__()

        # Moving topograhpy
        self.add_attribute('dtopofiles',[])
        self.add_attribute('dt_max_dtopo', 1.e99)

    def write(self, data_source='setrun.py', out_file='dtopo.data'):

        # Moving topography settings
        self.open_data_file(out_file, data_source)
        mdtopofiles = len(self.dtopofiles)
        self.data_write(value=mdtopofiles,alt_name='mdtopofiles')
        self.data_write()
        for tfile in self.dtopofiles:

            if len(tfile) == 4:
                w = '\n  *** WARNING: dtopofile specs changed in v5.8.0 -- ' + \
                      'Flag level info now ignored'
                warnings.warn(w, UserWarning)
                tfile = [tfile[0], tfile[-1]]  # drop minlevel,maxlevel
            elif len(tfile) == 2:
                pass   # now expect only dtopo_type, filename
            else:
                raise ValueError('Unexpected len(tfile) = %i' % len(tfile))

            # if path is relative in setrun, assume it's relative to the
            # same directory that out_file comes from
            fname = os.path.abspath(os.path.join(os.path.dirname(out_file),tfile[-1]))
            self._out_file.write("\n'%s' \n" % fname)
            self._out_file.write("%3i   # dtopo_type\n" % tfile[0])
        self.data_write()
        self.data_write(value=self.dt_max_dtopo,alt_name='dt_max_dtopo')
        self.close_data_file()


    def read(self, path="dtopo.data", force=False):
        r"""Read a dtopography data file."""

        print(self.dtopofiles)

        with open(os.path.abspath(path), 'r') as data_file:

            file_name = None

            # Forward to first parameter
            for line in data_file:

                # Regular parameter setting
                if "=:" in line:
                    value, tail = line.split("=:")
                    varname = tail.split()[0]

                    if varname == "mdtopofiles":
                        num_dtopo_files = int(value)
                    elif varname == "dt_max_dtopo":
                        self.dt_max_dtopo = float(value)

                # Assume this is the second line of a record, complete and add
                # to dtopofiles list
                elif file_name is not None:
                    base_values = [int(value) for value in line.split()]
                    base_values.append(file_name)
                    self.dtopofiles.append(base_values)
                    file_name = None

                # Non-empty line, assume start of dtopo_file record
                elif line[0] == "'":
                    file_name = line.strip()[1:-1]

        # Check to make sure we have all the dtopofiles
        if len(self.dtopofiles) != num_dtopo_files:
            raise IOError("The number of dtopo files specified does not equal ",
                          "the number found.")



class ForceDry(clawpack.clawutil.data.ClawData):

    def __init__(self):
        r"""
        A single force_dry array and associated data
        """

        super(ForceDry,self).__init__()
        self.add_attribute('tend',None)
        self.add_attribute('fname','')


class QinitData(clawpack.clawutil.data.ClawData):

    def __init__(self):

        super(QinitData,self).__init__()

        # Qinit data
        self.add_attribute('qinit_type',0)
        self.add_attribute('qinitfiles',[])
        self.add_attribute('variable_eta_init',False)
        self.add_attribute('force_dry_list',[])
        self.add_attribute('num_force_dry',0)

    def write(self,data_source='setrun.py', out_file='qinit.data'):

        # Initial perturbation
        self.open_data_file(out_file, data_source)
        self.data_write('qinit_type')

        # Perturbation requested
        if self.qinit_type == 0:
            pass
        else:
            # Check to see if each qinit file is present and then write the data
            for tfile in self.qinitfiles:

                if len(tfile) == 3:
                    w = '\n  *** WARNING: qinit specs changed in v5.8.0 -- ' + \
                          'Flag level info now ignored'
                    warnings.warn(w, UserWarning)
                    tfile = [tfile[-1]]  # drop minlevel,maxlevel
                elif len(tfile) == 1:
                    pass  # now expect only filename
                else:
                    raise ValueError('Unexpected len(tfile) = %i' % len(tfile))

                # if path is relative in setrun, assume it's relative to the
                # same directory that out_file comes from
                fname = os.path.abspath(os.path.join(os.path.dirname(out_file),tfile[-1]))
                self._out_file.write("\n'%s' \n" % fname)
        # else:
        #     raise ValueError("Invalid qinit_type parameter %s." % self.qinit_type)


        self.data_write('variable_eta_init')

        self.num_force_dry = len(self.force_dry_list)
        self.data_write('num_force_dry')

        for force_dry in self.force_dry_list:

            # if path is relative in setrun, assume it's relative to the
            # same directory that out_file comes from
            fname = os.path.abspath(os.path.join(os.path.dirname(out_file),\
                    force_dry.fname))
            self._out_file.write("\n'%s' \n" % fname)
            self._out_file.write("%.3f \n" % force_dry.tend)


        self.close_data_file()


# Storm data
class SurgeData(clawpack.clawutil.data.ClawData):
    r"""Data object describing storm surge related parameters"""

    # Provide some mapping between model names and integers
    storm_spec_dict_mapping = {"HWRF":-1,
                               None: 0,
                               'holland80': 1,
                               'holland08': 8,
                               'holland10': 2,
                               'CLE': 3,
                               'SLOSH': 4,
                               'rankine': 5,
                               'modified-rankine': 6,
                               'DeMaria': 7,
                               'willoughby': 9,
                              }
    storm_spec_not_implemented = ['CLE', 'willoughby']

    def __init__(self):
        super(SurgeData, self).__init__()

        # Source term controls
        self.add_attribute('wind_forcing', False)
        self.add_attribute('drag_law', 1)
        self.add_attribute('pressure_forcing', False)
        self.add_attribute('rotation_override', 0)

        # Algorithm parameters - Indexing is python based
        self.add_attribute("wind_index", 4)
        self.add_attribute("pressure_index", 6)
        self.add_attribute("display_landfall_time", False)

        # AMR parameters
        self.add_attribute('wind_refine', [20.0,40.0,60.0])
        self.add_attribute('R_refine', [60.0e3,40e3,20e3])

        # Storm parameters
        self.add_attribute('storm_type', None)  # Backwards compatibility
        self.add_attribute('storm_specification_type', 0) # Type of parameterized storm
        self.add_attribute("storm_file", None) # File(s) containing data


    def write(self, out_file='surge.data', data_source="setrun.py"):
        """Write out the data file to the path given"""

        # print "Creating data file %s" % out_file
        self.open_data_file(out_file,data_source)

        self.data_write('wind_forcing', description='(Wind source term used)')
        self.data_write('drag_law', description='(Type of drag law to use)')
        self.data_write('pressure_forcing',
                        description="(Pressure source term used)")
        if isinstance(self.rotation_override, str):
            if self.rotation_override.lower() == "normal":
                self.rotation_override = 0
            elif "n" in self.rotation_override.lower():
                self.rotation_override = 1
            elif "s" in self.rotation_override.lower():
                self.rotation_override = 2
            else:
                raise ValueError("Unknown rotation_override specification.")
        else:
            self.rotation_override = int(self.rotation_override)
        self.data_write('rotation_override', 
                        description="(Override storm rotation)")
        self.data_write()

        self.data_write("wind_index", value=self.wind_index + 1,
                        description="(Index into aux array - fortran indexing)")
        self.data_write("pressure_index", value=self.pressure_index +  1,
                        description="(Index into aux array - fortran indexing)")
        self.data_write("display_landfall_time",
                        description='(Display time relative to landfall)')
        self.data_write()

        if isinstance(self.wind_refine, bool):
            if not self.wind_refine:
                self.data_write('wind_refine', value=False,
                                description='(Refinement ratios)')
        elif isinstance(self.wind_refine, type(None)):
            self.data_write('wind_refine', value=False,
                            description='(Refinement ratios)')
        else:
            self.data_write('wind_refine',description='(Refinement ratios)')
        if isinstance(self.R_refine, bool):
            if not self.R_refine:
                self.data_write('R_refine', value=False,
                                description='(Refinement ratios)')
        elif isinstance(self.R_refine, type(None)):
            self.data_write('R_refine', value=False,
                            description='(Refinement ratios)')
        else:
            self.data_write('R_refine', description='(Refinement ratios)')
        self.data_write()

        # Storm specification
        if self.storm_type is not None:
            self.storm_specification_type = self.storm_type
        if type(self.storm_specification_type) is not int:
            if self.storm_specification_type in         \
                    self.storm_spec_dict_mapping.keys():
                if self.storm_specification_type in     \
                    self.storm_spec_not_implemented:
                    raise NotImplementedError("%s has not been implemented."
                                %self.storm_specification_type)

                else:
                    self.data_write("storm_specification_type",
                                self.storm_spec_dict_mapping[
                                        self.storm_specification_type],
                                description="(Storm specification)")
            else:
                raise ValueError("Unknown storm specification type %s"
                                 % self.storm_specification_type)
        else:
            self.data_write("storm_specification_type",
                            description="(Storm specification)")
        self.data_write("storm_file", description='(Path to storm data)')

        self.close_data_file()


class FrictionData(clawpack.clawutil.data.ClawData):
    r"""Data class representing complex variable friction"""

    def __init__(self):
        r""""""

        super(FrictionData, self).__init__()

        # Variable friction support
        self.add_attribute('variable_friction', False)

        # Index where the variable friction field is stored (Python indexed)
        self.add_attribute('friction_index', 3)

        # Region support
        self.add_attribute('friction_regions', [])

        # File support
        self.add_attribute('friction_files', [])

    def write(self, out_file='friction.data', data_source='setrun.py'):

        self.open_data_file(out_file, data_source)

        self.data_write('variable_friction',
                        description="(method for setting variable friction)")
        self.data_write('friction_index', value=self.friction_index + 1,
                        description=("(Index into aux array ",
                                     "- fortran indexing)"))
        self.data_write()
        if self.variable_friction:
            # Region based friction
            self.data_write(value=len(self.friction_regions),
                            alt_name='num_friction_regions',
                            description="(Friction Regions)")
            self.data_write()
            for region in self.friction_regions:
                self.data_write(value=region[0], alt_name="lower")
                self.data_write(value=region[1], alt_name="upper")
                self.data_write(value=region[2], alt_name="depths")
                self.data_write(value=region[3],
                                alt_name="manning_coefficients")
                self.data_write()

            # File based friction
            self.data_write(value=len(self.friction_files),
                            alt_name='num_friction_files')
            for friction_file in self.friction_files:
                # if path is relative in setrun, assume it's relative to the
                # same directory that out_file comes from
                fname = os.path.abspath(os.path.join(os.path.dirname(out_file),friction_file))
                self._out_file.write("'%s' %s\n " % fname)

        self.close_data_file()


class MultilayerData(clawpack.clawutil.data.ClawData):
    r"""
    Multilayer SWE data object
    """

    def __init__(self):
        super(MultilayerData, self).__init__()

        # Physics parameters
        self.add_attribute('num_layers', 1)
        self.add_attribute('rho', [1025.0, 1028.0])
        self.add_attribute('eta', [0.0, -200.0])
        self.add_attribute('wave_tolerance', [1.e-1, 1.e-1])

        # Algorithm parameters
        self.add_attribute('eigen_method', 4)
        self.add_attribute('inundation_method', 2)
        self.add_attribute('check_richardson', True)
        self.add_attribute('richardson_tolerance', 0.95)
        self.add_attribute('layer_index', 8)

        # Need to adjust refinement module for this, dry_limit is in geodata
        self.add_attribute('wave_tolerance', [1e-1, 2e-1])
        self.add_attribute('dry_limit', False)

    def write(self, out_file='multilayer.data', datasource="setrun.py"):

        self.open_data_file(out_file, datasource)

        self.data_write('num_layers', description='(Number of layers)')
        self.data_write('eta',
                        description='(Initial top surface of each layer)')
        self.data_write('wave_tolerance',
                        description=('(Tolerance of surface perturbation per',
                                     ' layer, used for refinement criteria)'))
        self.data_write('layer_index', value=self.layer_index + 1,
                        description=("(Index into aux array -",
                                     " fortran indexing)"))
        self.data_write(None)
        self.data_write('check_richardson',
                        description="(Check Richardson number)")
        self.data_write('richardson_tolerance',
                        description='(Tolerance for Richardson number)')
        self.data_write('eigen_method',
                        description='(Method for calculating eigenspace)')
        self.data_write('inundation_method',
                        description=('(Method for calculating inundation ',
                                     'eigenspace)'))
        self.close_data_file()



class BoussData(clawpack.clawutil.data.ClawData):
    r"""
     data object for Boussinesq info in 2D geoclaw

    """
    def __init__(self):
        super(BoussData,self).__init__()

        self.add_attribute('bouss_equations',2)
        self.add_attribute('bouss_min_level', 1)
        self.add_attribute('bouss_max_level', 10)
        self.add_attribute('bouss_min_depth', 10.)
        self.add_attribute('bouss_solver', 3)
        self.add_attribute('bouss_tstart', 0.)

    def write(self,out_file='bouss.data',data_source='setrun.py'):

        self.open_data_file(out_file,data_source)
        self.data_write('bouss_equations', description='0=SWE, 1=MS, 2=SGN')
        self.data_write('bouss_min_level',
                        description='coarsest level to apply bouss')
        self.data_write('bouss_max_level',
                        description='finest level to apply bouss')
        self.data_write('bouss_min_depth',
                        description='depth to switch to SWE')
        self.data_write('bouss_solver', description='1=GMRES, 2=Pardiso, 3=PETSc')
        self.data_write('bouss_tstart', description='time to switch from SWE')

        self.close_data_file()


# ==================================
# data objects for 1d_classic code
# ==================================


#  Gauge data object removed, version from amrclaw works in 1d
#class GaugeData1D(clawpack.clawutil.data.ClawData):


class GridData1D(clawpack.clawutil.data.ClawData):
    r"""
    1D data object for grid info

    """
    def __init__(self):
        super(GridData1D,self).__init__()

        self.add_attribute('grid_type',0)
        self.add_attribute('fname_celledges',None)
        self.add_attribute('monitor_fgmax',False)
        self.add_attribute('monitor_runup',False)
        self.add_attribute('monitor_total_zeta',False)

    def write(self,out_file='grid.data',data_source='setrun.py'):

        self.open_data_file(out_file,data_source)

        self.data_write('grid_type')
        if self.grid_type == 2:
            if self.fname_celledges is None:
                self.fname_celledges = 'celledges.txt'
                print('*** grid_type ==2 and fname_celledges not specified,')
                print('*** using celledges.txt')
            # if path is relative in setrun, assume it's relative to the
            # same directory that out_file comes from
            fname = os.path.abspath(os.path.join(os.path.dirname(out_file),
                                    self.fname_celledges))
            self._out_file.write("\n'%s'   =: fname_celledges\n " % fname)

        self._out_file.write("\n%s   =: monitor_fgmax" \
                             % str(self.monitor_fgmax)[0])
        self._out_file.write("\n%s   =: monitor_runup" \
                             % str(self.monitor_runup)[0])
        self._out_file.write("\n%s   =: monitor_total_zeta" \
                             % str(self.monitor_total_zeta)[0])
        self.close_data_file()

    def read(self, path, force=False):
        with open(os.path.abspath(path), 'r') as data_file:
            for line in data_file:
                if "=:" in line:
                    value, tail = line.split("=:")
                    varname = tail.split()[0]
                    if varname == 'grid_type':
                        self.grid_type = int(value)
                    elif varname == 'fname_celledges':
                        self.fname_celledges = value.strip()


class BoussData1D(clawpack.clawutil.data.ClawData):
    r"""
    1D data object for Boussinesq info

    """
    def __init__(self):
        super(BoussData1D,self).__init__()

        self.add_attribute('bouss_equations',2)
        self.add_attribute('bouss_min_depth',20.)

    def write(self,out_file='bouss.data',data_source='setrun.py'):

        self.open_data_file(out_file,data_source)

        self.data_write('bouss_equations')
        self.data_write('bouss_min_depth')

        self.close_data_file()






import os, sys
from clawpack.visclaw import geoplot
from clawpack.geoclaw.nonuniform_grid_tools import make_mapc2p
import numpy

fname_celledges = os.path.abspath('celledges.data')

fname = '_output/fgmax.txt'
try:
    d = numpy.loadtxt(fname)
    xmax = d[:,0]
    Bmax = d[:,1]
    hmax = d[:,2]
    etamax = numpy.where(hmax>1e-3, hmax+Bmax, numpy.nan)
    jmax = numpy.where(hmax>1e-3)[0].max()
    print("run-in = %8.2f m,  run-up = %8.2f m" % (xmax[jmax],etamax[jmax]))
    print('Loaded hmax from ',fname)
except:
    xmax = None
    print('Failed to load ',fname)
    
#xmax = None # to suppress plotting max elevation as red curve 

xlimits = [-100e3,1e3]


def setplot(plotdata):

    plotdata.clearfigures()

    outdir1 = plotdata.outdir
    mapc2p1, mx_edge, xp_edge = make_mapc2p(fname_celledges)

    def fix_layout(current_data):
        from pylab import tight_layout
        tight_layout()

    def add_etamax(current_data):
        from pylab import plot
        from clawpack.visclaw.legend_tools import add_legend
        if xmax is not None:
            plot(xmax, etamax, 'r')
            add_legend(['max eta over simulation','surface elevation eta'],
                   ['r','b'], framealpha=1)
        
    plotfigure = plotdata.new_plotfigure(name='domain', figno=0)
    plotfigure.kwargs = {'figsize':(8,7)}
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.axescmd = 'subplot(311)'
    plotaxes.xlimits = xlimits
    plotaxes.ylimits = [-3,10]
    plotaxes.title = 'Surface displacement at time h:m:s'
    plotaxes.grid = True

    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = geoplot.surface
    plotitem.color = 'b'
    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p1

    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    #plotitem.show = False
    plotitem.plot_var = geoplot.topo
    plotitem.color = 'g'
    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p1

    plotaxes = plotfigure.new_plotaxes()
    plotaxes.axescmd = 'subplot(312)'
    plotaxes.xlimits = xlimits
    plotaxes.ylimits = [-1.0,2.0]
    plotaxes.title = 'Velocity at time h:m:s'
    plotaxes.grid = True
    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p1

    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = geoplot.u_velocity
    plotitem.color = 'b'
    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p1

    plotaxes = plotfigure.new_plotaxes()
    plotaxes.axescmd = 'subplot(313)'
    plotaxes.xlimits = xlimits
    plotaxes.ylimits = [-5000,500]
    plotaxes.title = 'Full depth at time h:m:s'
    plotaxes.grid = True
    plotaxes.afteraxes = fix_layout
    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p1
    
    plotitem = plotaxes.new_plotitem(plot_type='1d_fill_between')
    #plotitem.show = False
    plotitem.plot_var = geoplot.surface
    plotitem.plot_var2 = geoplot.topo
    plotitem.color = [.5,.5,1]
    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p1

    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    #plotitem.show = False
    plotitem.plot_var = geoplot.topo
    plotitem.color = 'g'
    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p1

    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = geoplot.surface
    plotitem.color = 'b'
    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p1


    #----------

    plotfigure = plotdata.new_plotfigure(name='shore', figno=1)
    plotfigure.kwargs = {'figsize':(10,4)}
    #plotfigure.show = False
    
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = [-1500,500]
    plotaxes.ylimits = [-10,25]
    plotaxes.title = 'Zoom around shore at time h:m:s'
    plotaxes.grid = True
    plotaxes.afteraxes = add_etamax

    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.show = False
    plotitem.plot_var = geoplot.surface

    plotitem = plotaxes.new_plotitem(plot_type='1d_fill_between')
    plotitem.plot_var = geoplot.surface
    plotitem.plot_var2 = geoplot.topo
    plotitem.color = [.5,.5,1]
    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p1

    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = geoplot.surface
    plotitem.color = 'b'
    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p1

    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = geoplot.topo
    plotitem.color = 'g'
    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p1



    #-----------------------------------------
    # Figures for gauges
    #-----------------------------------------
    plotfigure = plotdata.new_plotfigure(name='eta', figno=300, \
                                         type='each_gauge')
    plotfigure.clf_each_gauge = True

    plotaxes = plotfigure.new_plotaxes()
    plotaxes.time_scale = 1/3600.  # convert seconds to hours
    plotaxes.time_label = 'time (hours) post-quake'
    plotaxes.xlimits = 'auto'
    #plotaxes.ylimits = [-2,2]
    plotaxes.title = 'Surface elevation eta'
    plotaxes.grid = True
    
    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = 2  # eta
    plotitem.plotstyle = 'b-'


    #-----------------------------------------

    # Parameters used only when creating html and/or latex hardcopy
    # e.g., via pyclaw.plotters.frametools.printframes:
    
    plotdata.printfigs = True                # print figures
    plotdata.print_format = 'png'            # file format
    plotdata.print_framenos = 'all'          # list of frames to print
    plotdata.print_gaugenos = 'all'          # list of gauges to print
    plotdata.print_fignos = 'all'            # list of figures to print
    plotdata.html = True                     # create html files of plots?
    plotdata.html_homelink = '../README.html'   # pointer for top of index
    plotdata.latex = True                    # create latex file of plots?
    plotdata.latex_figsperline = 2           # layout of plots
    plotdata.latex_framesperline = 1         # layout of plots
    plotdata.latex_makepdf = False           # also run pdflatex?
    plotdata.parallel = True                 # make multiple frame png's at once

    return plotdata



#!/usr/bin/env python
# encoding: utf-8
    
from __future__ import absolute_import
def acoustics(iplot=False,htmlplot=False,outdir='./_output'):
    """
    This example solves the 1-dimensional acoustics equations in a homogeneous
    medium.
    """
    import numpy as np
    from clawpack import riemann
    from clawpack import pyclaw

    solver = pyclaw.ClawSolver1D(riemann.acoustics_1D)

    solver.bc_lower[0] = pyclaw.BC.wall
    solver.bc_upper[0] = pyclaw.BC.wall

    solver.cfl_desired = 1.0
    solver.cfl_max     = 1.0

    x = pyclaw.Dimension(0.0,1.0,200,name='x')
    domain = pyclaw.Domain(x)
    num_eqn = 2
    state = pyclaw.State(domain,num_eqn)

    # Set problem-specific variables
    rho = 1.0
    bulk = 1.0
    state.problem_data['rho']=rho
    state.problem_data['bulk']=bulk
    state.problem_data['zz']=np.sqrt(rho*bulk)
    state.problem_data['cc']=np.sqrt(bulk/rho)

    # Set the initial condition
    xc = domain.grid.x.centers
    state.q[0,:] = np.cos(2*np.pi*xc)
    state.q[1,:] = 0.
    
    # Set up the controller
    claw = pyclaw.Controller()
    claw.solution = pyclaw.Solution(state, domain)
    claw.solver = solver
    claw.outdir = outdir
    claw.num_output_times = 40
    claw.tfinal = 2.0

    return claw

if __name__=="__main__":
    from clawpack.pyclaw.util import run_app_from_main
    output = run_app_from_main(acoustics)




from __future__ import print_function
from clawpack.geoclaw import topotools
from clawpack.geoclaw.data import Rearth  # radius of earth
from interp import pwcubic
from clawpack.clawutil.data import ClawData
import numpy as np
from matplotlib import pylab as plt

from mapper import latlong
import maketopo

probdata = ClawData()
probdata.read('setprob.data', force=True)
theta_island = probdata.theta_island
print("theta_island = ",theta_island)


(xisland,yisland) = latlong(1600.e3, theta_island, 40., Rearth)


def ocean():
    dmax = 1650.e3
    dp = np.linspace(0., dmax, 3301)
    zp = maketopo.shelf1(dp) 
    plt.clf()
    plt.plot(dp*1e-3,zp,'k-',linewidth=3)
    plt.fill_between(dp*1e-3,zp,0.,where=(zp<0.), color=[0,0.5,1])
    plt.xlim([0.,1650])
    plt.ylim([-4500.,500])
    plt.title("Topography as function of radius",fontsize=18)
    plt.xlabel("kilometers from center",fontsize=15)
    plt.ylabel("meters",fontsize=15)
    plt.xticks(fontsize=15)
    plt.yticks(fontsize=15)
    plt.savefig('topo1.png')
    #plt.savefig('topo1.tif')
    print("Cross-section plot in topo1.png")

    plt.xlim([1500,1650])
    plt.ylim([-200.,20])
    plt.title("Topography of shelf and beach",fontsize=18)
    plt.xlabel("kilometers from center",fontsize=15)
    plt.ylabel("meters",fontsize=15)
    plt.xticks(fontsize=15)
    plt.yticks(fontsize=15)
    plt.savefig('topo2.png')
    #plt.savefig('topo2.tif')
    print("Zoom near shore in topo2.png")


def with_island():
    dmax = 1650.e3
    dp = np.linspace(1500e3, dmax, 2000)
    zp = maketopo.shelf1(dp) + maketopo.island1(abs(dp-1600e3))

    plt.clf()
    plt.plot(dp*1e-3,zp,'k-',linewidth=3)
    plt.fill_between(dp*1e-3,zp,0.,where=(zp<0.), color=[0,0.5,1])

    plt.xlim([1500,1650])
    plt.ylim([-200.,30])
    plt.title("Cross-section through island center",fontsize=18)
    plt.xlabel("kilometers from center",fontsize=15)
    plt.ylabel("meters",fontsize=15)
    plt.xticks(fontsize=15)
    plt.yticks(fontsize=15)
    plt.savefig('topo3.png')
    #plt.savefig('topo3.tif')
    print("Cross-section through island in topo3.png")

if __name__=="__main__":
    ocean()
    with_island()




""" 
Set up the plot figures, axes, and items to be done for each frame.

This module is imported by the plotting routines and then the
function setplot is called to set the plot parameters.
    
""" 

import numpy as np
import matplotlib.pyplot as plt

from clawpack.geoclaw import topotools

try:
    TG32412 = np.loadtxt('32412_notide.txt')
except:
    print("*** Could not load DART data file")

#--------------------------
def setplot(plotdata=None):
#--------------------------
    
    """ 
    Specify what is to be plotted at each frame.
    Input:  plotdata, an instance of pyclaw.plotters.data.ClawPlotData.
    Output: a modified version of plotdata.
    
    """ 


    from clawpack.visclaw import colormaps, geoplot
    from numpy import linspace

    if plotdata is None:
        from clawpack.visclaw.data import ClawPlotData
        plotdata = ClawPlotData()


    plotdata.clearfigures()  # clear any old figures,axes,items data
    plotdata.format = 'ascii'      # 'ascii', 'binary', 'netcdf'


    # To plot gauge locations on pcolor or contour plot, use this as
    # an afteraxis function:

    def addgauges(current_data):
        from clawpack.visclaw import gaugetools
        gaugetools.plot_gauge_locations(current_data.plotdata, \
             gaugenos='all', format_string='ko', add_labels=True)
    

    # To add title with time format hours:minutes:seconds...

    def title_hours(current_data):
        from pylab import title, mod
        t = current_data.t
        hours = int(t/3600.)
        tmin = mod(t,3600.)
        min = int(tmin/60.)
        tsec = mod(tmin,60.)
        sec = int(mod(tmin,60.))
        timestr = '%s:%s:%s' % (hours,str(min).zfill(2),str(sec).zfill(2))
        title('%s after earthquake' % timestr)



    #-----------------------------------------
    # Figure for surface
    #-----------------------------------------
    plotfigure = plotdata.new_plotfigure(name='Surface', figno=0)
    plotfigure.kwargs = {'figsize':(12,6)}

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes('pcolor')
    plotaxes.axescmd = 'subplot(121)'
    plotaxes.title = 'Surface'
    plotaxes.scaled = True

    def fixup(current_data):
        addgauges(current_data)
        title_hours(current_data)

    plotaxes.afteraxes = fixup

    # Water
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = geoplot.surface_or_depth
    plotitem.pcolor_cmap = geoplot.tsunami_colormap
    plotitem.pcolor_cmin = -0.2
    plotitem.pcolor_cmax = 0.2
    plotitem.add_colorbar = False
    plotitem.amr_celledges_show = [0,0,0]
    plotitem.patchedges_show = 1

    # Land
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = geoplot.land
    plotitem.pcolor_cmap = geoplot.land_colors
    plotitem.pcolor_cmin = 0.0
    plotitem.pcolor_cmax = 100.0
    plotitem.add_colorbar = False
    plotitem.amr_celledges_show = [0,0,0]
    plotitem.patchedges_show = 0
    plotaxes.xlimits = [-120,-60]
    plotaxes.ylimits = [-60,0]

    # add contour lines of bathy if desired:
    plotitem = plotaxes.new_plotitem(plot_type='2d_contour')
    plotitem.show = False
    plotitem.plot_var = geoplot.topo
    plotitem.contour_levels = linspace(-3000,-3000,1)
    plotitem.amr_contour_colors = ['y']  # color on each level
    plotitem.kwargs = {'linestyles':'solid','linewidths':2}
    plotitem.amr_contour_show = [1,0,0]  
    plotitem.celledges_show = 0
    plotitem.patchedges_show = 0


    #-----------------------------------------
    # Figure for adjoint flagging
    #-----------------------------------------


    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes('adjoint')
    plotaxes.axescmd = 'subplot(122)'
    plotaxes.scaled = True
    plotaxes.title = 'Adjoint flag'

    def fixup(current_data):
        from pylab import title
        addgauges(current_data)
        title('Adjoint flagging indicator')

    plotaxes.afteraxes = fixup


    def masked_inner_product(current_data):
        from numpy import ma
        aux = current_data.aux
        tol = 1e-15
        soln = ma.masked_where(aux[3,:,:] < tol, aux[3,:,:])
        return soln

    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = masked_inner_product
    plotitem.pcolor_cmap = colormaps.white_red
    plotitem.pcolor_cmin = 0.0
    plotitem.pcolor_cmax = 0.0005
    #plotitem.pcolor_cmax = 0.00001 # use for adjoint-error flagging

    plotitem.add_colorbar = False   # doesn't work when adjoint all masked
    plotitem.colorbar_shrink = 0.75
    plotitem.amr_celledges_show = [0,0,0]
    plotitem.amr_data_show = [1,1,0] # inner product not computed on finest level
    plotitem.patchedges_show = 0

    # Land
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = geoplot.land
    plotitem.pcolor_cmap = geoplot.land_colors
    plotitem.pcolor_cmin = 0.0
    plotitem.pcolor_cmax = 100.0
    plotitem.add_colorbar = False
    plotitem.amr_celledges_show = [0,0,0]
    plotitem.patchedges_show = 0
    plotaxes.xlimits = [-120,-60]
    plotaxes.ylimits = [-60,0]


    #-----------------------------------------
    # Figures for gauges
    #-----------------------------------------
    plotfigure = plotdata.new_plotfigure(name='Surface at gauges', figno=300, \
                    type='each_gauge')
    plotfigure.clf_each_gauge = True

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = 'auto'
    plotaxes.ylimits = 'auto'
    plotaxes.title = 'Surface'

    # Plot surface as blue curve:
    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = 3
    plotitem.plotstyle = 'b-'

    # Plot topo as green curve:
    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.show = False

    def gaugetopo(current_data):
        q = current_data.q
        h = q[0,:]
        eta = q[3,:]
        topo = eta - h
        return topo
        
    plotitem.plot_var = gaugetopo
    plotitem.plotstyle = 'g-'

    def add_zeroline(current_data):
        from pylab import plot, legend, xticks, floor, axis, xlabel
        t = current_data.t 
        gaugeno = current_data.gaugeno

        if gaugeno == 32412:
            try:
                plot(TG32412[:,0], TG32412[:,1], 'r')
                legend(['GeoClaw','Obs'],loc='lower right')
            except: pass
            axis((0,t.max(),-0.3,0.3))

        plot(t, 0*t, 'k')
        n = int(floor(t.max()/3600.) + 2)
        xticks([3600*i for i in range(n)], ['%i' % i for i in range(n)])
        xlabel('time (hours)')

    plotaxes.afteraxes = add_zeroline



    #-----------------------------------------
    
    # Parameters used only when creating html and/or latex hardcopy
    # e.g., via pyclaw.plotters.frametools.printframes:

    plotdata.printfigs = True                # print figures
    plotdata.print_format = 'png'            # file format
    plotdata.print_framenos = 'all'          # list of frames to print
    plotdata.print_gaugenos = 'all'          # list of gauges to print
    plotdata.print_fignos = 'all'            # list of figures to print
    plotdata.html = True                     # create html files of plots?
    plotdata.html_homelink = '../README.html'   # pointer for top of index
    plotdata.latex = True                    # create latex file of plots?
    plotdata.latex_figsperline = 2           # layout of plots
    plotdata.latex_framesperline = 1         # layout of plots
    plotdata.latex_makepdf = False           # also run pdflatex?
    plotdata.parallel = True                 # make multiple frame png's at once

    return plotdata




"""
Plot results at three gauges and compare with experimental data, obtained from
    https://nctr.pmel.noaa.gov/benchmark/Solitary_wave/

If you run this after running GeoClaw using the Madsen-Sorenson option with:
    rundata.bouss_data.boussEquations = 1
then you can compare the figure generated to Figure 4 in the BoussClaw paper
    http://dx.doi.org/10.1016/j.coastaleng.2017.01.005


See also 
    compare_BoussSWE.py
which runs the code with various settings and plots the results together
using plot_gauges from this module.

"""

from pylab import *
import clawpack.pyclaw.gauges as gauges

def plot_gauges(outdirs=[('_output', 'Simulation', 'b')],
                fname_figure='GaugeComparison.png'):

    d = loadtxt('experimental_data/ts3b.txt',skiprows=6)
    tg = d[:,0]
    g5 = d[:,2]
    g7 = d[:,4]
    g8 = d[:,5]

    figure(400, figsize=(8,8))
    clf()

    subplot(311)
    gaugeno = 5

    plot(tg-tg[0], g5, 'r--', label='Experiment')
    
    for (outdir,outdir_label,outdir_color) in outdirs:
        gauge = gauges.GaugeSolution(gaugeno, outdir)
        t = gauge.t
        eta = gauge.q[2,:]
        plot(t, eta, color=outdir_color, label=outdir_label)


    xlim(0,25)
    ylim(-0.01,0.08)
    legend(loc='upper right')
    grid(True)
    xlabel('')
    ylabel('Surface (m)')
    title('Gauge %i' % gaugeno)

    subplot(312)
    gaugeno = 7
    
    plot(tg-tg[0], g7, 'r--', label='Experiment')
    
    for (outdir,outdir_label,outdir_color) in outdirs:
        gauge = gauges.GaugeSolution(gaugeno, outdir)
        t = gauge.t
        eta = gauge.q[2,:]
        plot(t, eta, color=outdir_color, label=outdir_label)
        

    xlim(0,25)
    ylim(-0.01,0.08)
    legend(loc='upper right')
    grid(True)
    xlabel('')
    ylabel('Surface (m)')
    title('Gauge %i' % gaugeno)

    subplot(313)
    gaugeno = 8

    plot(tg-tg[0], g8, 'r--', label='Experiment')
    
    for (outdir,outdir_label,outdir_color) in outdirs:
        gauge = gauges.GaugeSolution(gaugeno, outdir)
        t = gauge.t
        eta = gauge.q[2,:]
        plot(t, eta, color=outdir_color, label=outdir_label)


    xlim(0,25)
    ylim(-0.01,0.08)
    legend(loc='upper right')
    grid(True)
    xlabel('')
    ylabel('Surface (m)')
    title('Gauge %i' % gaugeno)

    tight_layout()

    if fname_figure is not None:
        savefig(fname_figure, bbox_inches='tight')
        print('Created %s' % fname_figure)

if __name__=='__main__':

    plot_gauges(outdirs=[('_output', 'Simulation', 'b')],
                fname_figure='GaugeComparison.png')




def configuration(parent_package='',top_path=None):
    import os

    from numpy.distutils.misc_util import Configuration

    config = Configuration('iso_c_advection', parent_package, top_path)

    print("You will probably want to run make manually here")

    return config

if __name__ == '__main__':
    from numpy.distutils.core import setup
    setup(**configuration().todict())



from __future__ import absolute_import
from __future__ import print_function
from numpy import random, empty,zeros
#from numpy import *
from FRS import vec_rp, pw_rp
from time import clock
from pylab import plot,  figure
from six.moves import range




class RiemannSolver:
  

    def __init__(self, timeSteps, mwaves, mx, meqn, maux, q = None, aux = None):
       
        self.timeSteps = timeSteps
        self.mwaves = mwaves
        self.mx = mx
        self.meqn = meqn
        self.maux = maux
        if q is not None:
            self.q = q
        if aux is not None:
            self.aux = aux

    def solveVectorized(self, timer = None):
       
        if timer is not None:
            print("Solving... vectorized")
            start = clock()

        for counter in range(self.timeSteps):
            waves, s = vec_rp(self.mwaves,self.q[:,0:self.mx-1],self.q[:,1:self.mx],self.aux[:,0:self.mx-1],self.aux[:,1:self.mx])

        if timer is not None:
            end = clock()
            self.elapsedTime = end-start
            print("elapsed time: ", self.elapsedTime, " seconds")

        return waves, s




    def solvePointwize(self, timer = None):

        timeSteps, mwaves, mx, meqn, maux, q , aux = self.timeSteps, self.mwaves, self.mx, self.meqn, self.maux, self.q , self.aux
        waves = zeros((meqn, mwaves, mx))
        s = zeros((mwaves, mx))
        if timer is not None:
            print("Solving... point-wize")
            start = clock()

        for counter in range(timeSteps):
            for i in range(1,mx-1):  #from 1 to mx-1 unlike fortran (from 1 to mx-2 means including mx-2)
                waves[:,:,i], s[:,i] = pw_rp(mwaves,q[:,i],q[:,i+1],aux[:,i],aux[:,i+1]) # instead of pw_rp(mwaves,q[:,i-1],q[:,i],aux[:,i-1],aux[:,i]) due to the
                                                                                         # differences in indexing among fortran and python

        if timer is not None:
            end = clock()
            self.elapsedTime = end-start
            print("elapsed time: ", self.elapsedTime, " seconds")

        return waves, s
        

        

if __name__ == "__main__":
    rs = RiemannSolver( 1, 2, 2**10, 2, 2)
    
    rs.q = random.random((rs.meqn,rs.mx))   
    rs.aux = random.random((rs.maux,rs.mx))

    
    waves1, s1  = rs.solveVectorized(timer = True)

    
    waves2, s2  = rs.solvePointwize(timer = True)
   




"""
Module to create topo and qinit data files for this example.
"""

from __future__ import absolute_import
from __future__ import print_function
from clawpack.geoclaw.topotools import Topography

from clawpack.geoclaw import dtopotools
import numpy
import matplotlib.pyplot as plt
import os


def maketopo():
    """
    Output topography file for the entire domain
    """
    nxpoints=201
    nypoints=201
    xlower = -10e0
    xupper= 10e0
    ylower = -10e0
    yupper= 10e0
    outfile= "topo1.topotype2"
    topography = Topography(topo_func=topo)
    topography.x = numpy.linspace(xlower,xupper,nxpoints)
    topography.y = numpy.linspace(ylower,yupper,nypoints)
    topography.write(outfile, topo_type=2, Z_format="%22.15e")
    dx = (xupper-xlower)/(nxpoints-1)
    dy = (yupper-ylower)/(nypoints-1)
    print("==> topo1 has dx = %g, dy = %g" % (dx,dy))

def maketopo2():
    """
    Another topo file for testing
    """
    nxpoints=21
    nypoints=51
    xlower = -0.5
    xupper= -0.3
    ylower = -0.1
    yupper= 0.4
    outfile= "topo2.topotype2"
    topography = Topography(topo_func=topo2)
    topography.x = numpy.linspace(xlower,xupper,nxpoints)
    topography.y = numpy.linspace(ylower,yupper,nypoints)
    topography.write(outfile, topo_type=2, Z_format="%22.15e")
    dx = (xupper-xlower)/(nxpoints-1)
    dy = (yupper-ylower)/(nypoints-1)
    print("==> topo2 has dx = %g, dy = %g" % (dx,dy))

h0 = 1000.

def topo(x,y):
    #z = -10*numpy.ones(x.shape)
    z = -h0*(1 + 0.5*numpy.cos(x-y))
    return z

def topo2(x,y):
    #z = -15*numpy.ones(x.shape)
    z = -h0*(1. + numpy.exp(x+y))
    return z


def read_fault(fname_subfaults, plotfig=None):
    """
    Test data
    """

    input_units = {'slip': 'm', 'depth': 'km', 'length': 'km', 'width': 'km'}

    fault = dtopotools.CSVFault()
    fault.read(fname_subfaults, input_units=input_units, 
                coordinate_specification="top center")
    fault.rupture_type = 'dynamic'
                    
    if plotfig:
        plt.figure(plotfig)
        fault.plot_subfaults(slip_color=True,plot_rake=True)
        
    return fault


def make_dtopo1(plotfig=None):
    """
    Test data.
    """

    fname_subfaults = 'dtopo1.csv'
    fault = read_fault(fname_subfaults)

    dtopo_fname = fname_subfaults.split('.')[0] + '.tt3'

    print("Using Okada model to create %s " % dtopo_fname)

    # Needed for extent of dtopo file:
    xlower = -0.4
    xupper = 0.6
    ylower = -0.4
    yupper = 0.4

    # number of grid points in dtopo file
    mx = 151
    my = 121

    dx = (xupper-xlower)/(mx-1)
    dy = (yupper-ylower)/(my-1)
    print("==> dtopo1 has dx = %g, dy = %g" % (dx,dy))
    x = numpy.linspace(xlower,xupper,mx)
    y = numpy.linspace(ylower,yupper,my)

    tfinal = 1.
    times = numpy.linspace(0.,tfinal,25)
    dtopo = fault.create_dtopography(x,y,times=times)
    dtopo.write(dtopo_fname,3)


    if plotfig:
        plt.figure(plotfig)
        # plot final deformation:
        dtopo.plot_dz_colors(tfinal, cmax_dz=16,dz_interval=1)
        # make animation as html:
        dtopo.animate_dz_colors(times, cmax_dz=16,dz_interval=1,style='html')

    return dtopo


def make_dtopo2(plotfig=None):
    """
    Test data.
    """

    fname_subfaults = 'dtopo2.csv'
    fault = read_fault(fname_subfaults)

    dtopo_fname = fname_subfaults.split('.')[0] + '.tt3'

    print("Using Okada model to create %s " % dtopo_fname)

    # Needed for extent of dtopo file:
    xlower = -0.9
    xupper = 0.1
    ylower = -0.4
    yupper = 0.4

    # number of grid points in dtopo file
    mx = 201
    my = 161

    dx = (xupper-xlower)/(mx-1)
    dy = (yupper-ylower)/(my-1)
    print("==> dtopo2 has dx = %g, dy = %g" % (dx,dy))


    x = numpy.linspace(xlower,xupper,mx)
    y = numpy.linspace(ylower,yupper,my)

    tfinal = 1.2
    times = numpy.linspace(0.5,tfinal,25)
    dtopo = fault.create_dtopography(x,y,times=times)
    dtopo.write(dtopo_fname,3)


    if plotfig:
        plt.figure(plotfig)
        # plot final deformation:
        dtopo.plot_dz_colors(tfinal, cmax_dz=16,dz_interval=1)
        # make animation as html:
        dtopo.animate_dz_colors(times, cmax_dz=16,dz_interval=1,style='html')

        return dtopo

if __name__=='__main__':
    maketopo()
    maketopo2()
    make_dtopo1()
    make_dtopo2()




def configuration(parent_package='',top_path=None):
    from numpy.distutils.misc_util import Configuration
    config = Configuration('clawpack',parent_package,top_path)
    config.add_subpackage('clawutil',  subpackage_path='clawutil/src/python/clawutil')
    config.add_subpackage('riemann', subpackage_path='riemann/riemann')
    config.add_subpackage('visclaw',  subpackage_path='visclaw/src/python/visclaw')
    config.add_subpackage('pyclaw', subpackage_path='pyclaw/src/pyclaw')
    config.add_subpackage('petclaw', subpackage_path='pyclaw/src/petclaw')
    config.add_subpackage('forestclaw', subpackage_path='pyclaw/src/forestclaw')
    config.add_subpackage('classic',  subpackage_path='classic/src/python/classic')
    config.add_subpackage('amrclaw',  subpackage_path='amrclaw/src/python/amrclaw')
    config.add_subpackage('geoclaw',  subpackage_path='geoclaw/src/python/geoclaw')
    return config

if __name__ == '__main__':
    from numpy.distutils.core import setup
    setup(**configuration(top_path='').todict())



from . import stegoton
import numpy as np
from pytest import importorskip
import os

thisdir = os.path.dirname(__file__)
path_expected_sols = os.path.join(thisdir,'expected_sols.npy')
expected_sols_dict = np.load(path_expected_sols,allow_pickle=True).item()

def error(test_name,**kwargs):
    claw = stegoton.setup(tfinal=50.0, outdir=None, **kwargs)
    claw.run()
    strain_test = claw.frames[claw.num_output_times].state.get_q_global()[0].reshape([-1])
    dx = claw.solution.domain.grid.delta[0]
    assert test_name in expected_sols_dict.keys(), f"Test name {test_name} not found in {path_expected_sols}"
    strain_expected = expected_sols_dict[test_name]
    diff_L1 = dx*np.sum(np.abs(strain_test-strain_expected))
    return diff_L1

class TestStegoton1D:
    def test_fortran_classic(self):
        assert error(test_name='fortran_classic', kernel_language='Fortran', solver_type="classic")<1e-6

    def test_fortran_sharpclaw(self):
        assert error(test_name='fortran_sharpclaw', kernel_language='Fortran', solver_type="sharpclaw")<1e-6
    
    def test_python_classic(self):
        assert error(test_name='python_classic', kernel_language='Python', solver_type="classic")<1e-6
    
    def test_python_sharpclaw(self):
        assert error(test_name='python_sharpclaw', kernel_language='Python', solver_type="sharpclaw")<1e-6


#!/usr/bin/env python
# encoding: utf-8
r"""
Simple convenience script for setting up and running an interactive plotting
sesssion.
"""

from __future__ import absolute_import
import sys
import clawpack.visclaw.Iplotclaw as Iplotclaw

def run_iplotclaw(setplot='setplot.py',outdir='./_output'):
    ip = Iplotclaw.Iplotclaw(setplot=setplot,outdir=outdir)
    ip.plotloop()

if __name__ == '__main__':
    if len(sys.argv) > 1:
        run_iplotclaw(*sys.argv[1:])
    else:
        run_iplotclaw()




""" 
Set up the plot figures, axes, and items to be done for each frame.

This module is imported by the plotting routines and then the
function setplot is called to set the plot parameters.
    
""" 

#--------------------------
def setplot(plotdata=None):
#--------------------------
    
    """ 
    Specify what is to be plotted at each frame.
    Input:  plotdata, an instance of clawpack.visclaw.data.ClawPlotData.
    Output: a modified version of plotdata.
    
    """ 

    if plotdata is None:
        from clawpack.visclaw.data import ClawPlotData
        plotdata = ClawPlotData()

    plotdata.clearfigures()  # clear any old figures,axes,items data

    gamma = 1.4
    def pressure(current_data):
        q = current_data.q
        rho = q[0,:]
        u = q[1,:]/rho
        p = (gamma-1)*(q[2,:] - 0.5*rho*u**2)
        return p

    level_labels = ['Level 1','Level 2', 'Level 3', 'Level 4']
    level_colors = ['k','g','b','r']
    level_styles = '-'
    level_markers = ''

    def add_legend(current_data):
        try:
            from clawpack.visclaw import legend_tools
            legend_tools.add_legend(labels=level_labels, colors=level_colors,
                        markers=level_markers, linestyles=level_styles,
                        loc='upper right')
        except:
            print('legend_tools error')
            pass


    # Density and pressure plots
    # --------------------------

    plotfigure = plotdata.new_plotfigure(name='Density and Pressure', figno=1)
    plotfigure.kwargs = {'figsize':(10,5)}

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.axescmd = 'subplot(1,2,1)'   # left figure
    plotaxes.xlimits = [0,1]
    plotaxes.ylimits = [0,7]
    plotaxes.title = 'Density'
    #plotaxes.afteraxes = add_legend

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='1d_pwconst')
    plotitem.plot_var = 0
    plotitem.amr_color = level_colors
    plotitem.amr_plotstyle = level_styles
    plotitem.amr_data_show = [1]  # all levels


    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.axescmd = 'subplot(1,2,2)'   # right
    plotaxes.xlimits = [0,1]
    plotaxes.ylimits = [-10,1100]
    plotaxes.title = 'Pressure'
    plotaxes.afteraxes = add_legend

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='1d_pwconst')
    plotitem.plot_var = pressure
    plotitem.amr_color = level_colors
    plotitem.amr_plotstyle = level_styles
    plotitem.amr_data_show = [1]  # all levels

    #  Density and pressure plots, zoomed at final time
    # -------------------------------------------------

    plotfigure = plotdata.new_plotfigure(name='zoom view', figno=2)
    plotfigure.kwargs = {'figsize':(10,5)}

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.axescmd = 'subplot(1,2,1)'   # left figure
    plotaxes.xlimits = [0.5,0.9]
    plotaxes.ylimits = [0,7]
    plotaxes.title = 'Density'
    #plotaxes.afteraxes = add_legend

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='1d_pwconst')
    plotitem.plot_var = 0
    plotitem.amr_color = level_colors
    plotitem.amr_plotstyle = level_styles
    plotitem.amr_data_show = [1]  # all levels


    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.axescmd = 'subplot(1,2,2)'   # right
    plotaxes.xlimits = [0.5,0.9]
    plotaxes.ylimits = [-10,450]
    plotaxes.title = 'Pressure'
    plotaxes.afteraxes = add_legend

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='1d_pwconst')
    plotitem.plot_var = pressure
    plotitem.amr_color = level_colors
    plotitem.amr_plotstyle = level_styles
    plotitem.amr_data_show = [1]  # all levels

    
    #-----------------------------------------
    # Figures for gauges
    #-----------------------------------------
    plotfigure = plotdata.new_plotfigure(name='q', figno=300, \
                                         type='each_gauge')
    plotfigure.clf_each_gauge = True

    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = 'auto'
    plotaxes.ylimits = [-1,30]
    plotaxes.title = 'Density'
    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = 0
    plotitem.plotstyle = 'b-'


    # Parameters used only when creating html and/or latex hardcopy
    # e.g., via clawpack.visclaw.frametools.printframes:

    plotdata.printfigs = True                # print figures
    plotdata.print_format = 'png'            # file format
    plotdata.print_framenos = 'all'          # list of frames to print
    plotdata.print_fignos = 'all'            # list of figures to print
    plotdata.html = True                     # create html files of plots?
    plotdata.html_homelink = '../README.html'
    plotdata.latex = True                    # create latex file of plots?
    plotdata.latex_figsperline = 2           # layout of plots
    plotdata.latex_framesperline = 1         # layout of plots
    plotdata.latex_makepdf = False           # also run pdflatex?

    return plotdata

    
